package dans.GUI;

import static dans.GUI.NetworkSettingsTableModel.rowNames;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import javax.swing.table.AbstractTableModel;
import dans.network.NetworkObject;
import dans.network.NetworkObjectListener;
import dans.network.Processor;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Dan
 * @todo Refactor
 * @todo Add tool tips.
 */
public class AlgorithmsTableModel extends AbstractTableModel implements NetworkObjectListener  {
    private class DataRow {
        public Object port;
        public Object algorithm;
        
        public DataRow(Object p, Object a) {
            port = p;
            algorithm = a;
        }
        
        public Object getCol(int c) {
            return c == 0 ? port : algorithm;
        }
        
        public void setCol(int c, Object o) {
            if(c == 0) {
                port = o;
            } else {
                algorithm = o;
            }
        }
    }
    
    List<DataRow> rowData = new ArrayList<>();
    public static final String[] columnNames = {"Port", "Algorithm", "Remove"};
    private GUIHandler hgui;
    private mxProcessor mxp;
    
    private int lastPort = 0;
    
    public AlgorithmsTableModel(GUIHandler hgui, mxProcessor mxp) {
        this.mxp = mxp;
        
        for(Object o[] : mxp.getProcessor().dumpAlgorithms()) {
            rowData.add(new DataRow(o[0], o[1]));
        }
        rowData.add(new DataRow("<New Port>", "<New Algorithm>"));
        
        this.hgui = hgui;
        mxp.getProcessor().addListener(this, true);
    }
    
    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    @Override
    public String getColumnName(int column) {
        return columnNames[column];
    }

    @Override
    public int getRowCount() {
        return rowData.size();
    }

    @Override
    public Object getValueAt(int row, int column) {
        if(column >= 0 && column <= 1) {
            return rowData.get(row).getCol(column);
        }
        
        return null;
    }

    @Override
    public Class getColumnClass(int column) {
        if(column == 0) {
            return Integer.class;
        } else {
            return String.class;
        }
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        if(value instanceof String) {
            value = ((String)value).replaceAll(",", "");
        }
        
        if(column >= 0 && column <= 1) {
            DataRow r = rowData.get(row);

            if((r.getCol(column) == null && value != null) || !r.getCol(column).equals(value)) {
                if(column == 0 && value != null && !value.equals("<New Port>") && !value.equals("")) {
                    boolean okPort = true;
                    for(DataRow i : rowData) {
                        if(r != i && value.equals(i.port)) {
                            okPort = false;
                        }
                    }

                    if(okPort) {
                        int oldPort;
                        int newPort;
                        try {
                            oldPort = Integer.parseInt(r.port.toString());
                        } catch(NumberFormatException e) {
                            oldPort = -1;
                        }

                        r.port = value;
                        //fireTableCellUpdated(row, column);

                        try {
                            newPort = Integer.parseInt(r.port.toString());
                        } catch(NumberFormatException e) {
                            newPort = -1;
                        }

                        if(oldPort != newPort) {
                            if(r.algorithm != null && !r.algorithm.equals("<New Algorithm>") && !r.algorithm.equals("")) {
                                if(newPort >= 0){
                                    if(oldPort >= 0 && mxp.getProcessor().portInUse(oldPort)) {
                                        mxp.getProcessor().updatePort(oldPort, newPort, this);
                                    } else if(!mxp.getProcessor().portInUse(newPort)){
                                        mxp.getProcessor().addAlgorithm(newPort, r.algorithm.toString(), this);
                                        newRow();
                                    }
                                }
                            }
                        }
                    }
                } else if(column == 1 && value != null && !value.equals("<New Algorithm>") && !value.equals("")) {
                    if(!r.algorithm.equals(value)) {
                        r.algorithm = value;
                        //fireTableCellUpdated(row, column);

                        if(r.port!= null && !r.port.equals("<New Port>") && !r.port.equals("")) {
                            int port;
                            try {
                                port = Integer.parseInt(r.port.toString());
                            } catch(NumberFormatException e) {
                                port = -1;
                            }

                            if(port >= 0) {
                                if(mxp.getProcessor().portInUse(port)) {
                                    mxp.getProcessor().updateAlgorithm(port, r.algorithm.toString(), this);
                                } else {
                                    mxp.getProcessor().addAlgorithm(port, r.algorithm.toString(), this);
                                    newRow();
                                }
                            }
                        }
                    }
                }
            }

            fireTableCellUpdated(row, column);
        }
    }

    public void newRow() {
        rowData.add(new DataRow("<New Port>", "<New Algorithm>"));
        fireTableRowsInserted(0, rowData.size() - 1);
    }
    
    void removeRow(int row) {
        DataRow r = rowData.get(row);
        if(r != null) {
            if(r.port != null && !r.port.equals("<New Port>") && !r.port.equals("") && r.algorithm != null && !r.algorithm.equals("<New Algorithm>") && !r.algorithm.equals("")) {
                int port;
                try {
                    port = Integer.parseInt(r.port.toString());
                } catch(NumberFormatException e) {
                    port = -1;
                }
                
                if(port >= 0) {
                    mxp.getProcessor().removeAlgorithm(port, this);
                }
            }
            
            rowData.remove(row);
            fireTableRowsDeleted(0, rowData.size() - 1);
            hgui.warningCheck(mxp);
        }
    }
    
    @Override
    public boolean isCellEditable(int row, int column) {
        return true;
    } 

    @Override
    public void networkObjectChange(NetworkObject object, Object oldvalue, int eventID, Object caller) {
        if(eventID == Processor.EVENT_ALGORITHM_ADDED || eventID == Processor.EVENT_ALGORITHM_REMOVED || eventID == Processor.EVENT_ALGORITHM_UPDATED) {
            rowData.clear();
            for(Object o[] : mxp.getProcessor().dumpAlgorithms()) {
                rowData.add(new DataRow(o[0], o[1]));
            }
            rowData.add(new DataRow("<New Port>", "<New Algorithm>"));
            
            if(eventID == Processor.EVENT_ALGORITHM_ADDED) {
                fireTableRowsInserted(0, rowData.size() - 1);
            } else if(eventID == Processor.EVENT_ALGORITHM_REMOVED) {
                fireTableRowsDeleted(0, rowData.size() - 1);
            } else {
                fireTableRowsUpdated(0, rowData.size() - 1);
            }
            
            fireTableDataChanged();
        }
    }
    
    public void removeListener() {
        mxp.getProcessor().removeListener(this);
    }
}
