package dans.GUI;

import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxGraphModel.mxChildChange;
import com.mxgraph.model.mxGraphModel.mxGeometryChange;
import com.mxgraph.model.mxGraphModel.mxTerminalChange;
import com.mxgraph.model.mxGraphModel.mxValueChange;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.util.mxConstants;
import com.mxgraph.util.mxEvent;
import com.mxgraph.util.mxEventObject;
import com.mxgraph.util.mxEventSource.mxIEventListener;
import com.mxgraph.util.mxPoint;
import com.mxgraph.util.mxRectangle;
import com.mxgraph.util.mxUndoManager;
import com.mxgraph.util.mxUndoableEdit;
import com.mxgraph.util.mxUndoableEdit.mxUndoableChange;
import com.mxgraph.view.mxGraph;
import com.mxgraph.view.mxGraphSelectionModel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.Hashtable;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import com.mxgraph.swing.handler.mxRubberband;
import dans.util.Log;
import dans.network.Link;
import dans.network.NetworkState;
import dans.network.NetworkStateListener;
import dans.network.Processor;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author Dan
 */
public class GUI extends JFrame {
    private mxNetwork graph;
    private Object parent;
    private mxGraphComponent graphComponent;
    private mxUndoManager undoManager;
    private mxIEventListener undoHandler;
    
    private MenuBar menuBar;
    private ToolBar toolBar;
    private PropertiesTab propsTab;
    private LogWindow logWindow;
    private StatusBar statBar;
    private JSplitPane splitPaneHoriz;
    private JSplitPane splitPaneVert;
    
    private KeyboardHandler hkeyboard;
    private GUIHandler hgui;
    
    private mxRubberband bandselect;
    
    public GUI(NetworkState state){
        Log.debug("Creating GUI...");
        
        this.setTitle("DANS: " + state.getNetworkSetting("networkName"));
        this.setSize(1080, 700);
        this.setMinimumSize(new Dimension(390,225));
        this.setLocation(300, 200);
        
        hgui = new GUIHandler (this, state);
        
        Log.debug("Setting up JGraphX...");
        //JGraphX Setup
        graph = new mxNetwork();
        graph.setCellsSelectable(true);
        graph.setAllowDanglingEdges(false);
        graph.setCellsResizable(false);
        graph.setKeepEdgesInBackground(false);
        graph.setEdgeLabelsMovable(false);
        graph.setCellsBendable(false);
        parent = graph.getDefaultParent();
        graphComponent = new mxGraphComponent(graph) {
            @Override
            public boolean isPanningEvent(MouseEvent e) {
                return hgui.isPanningEvent(e);
            }
            
            @Override
            public boolean isToggleEvent(MouseEvent e) {
                return e.isControlDown() || e.isShiftDown();
            }
        };
        graphComponent.setConnectable(false);
        graphComponent.setPanning(true);
        bandselect = new mxRubberband(graphComponent);
        //graph.setMinimumGraphSize(new mxRectangle(0,0,2000,2000));
        //graphComponent.setMinimumSize(new Dimension(2000,2000));
        hgui.register(graphComponent);
        
        //Undo/redo Setup
        undoManager = new mxUndoManager();

        undoHandler = (Object source, mxEventObject evt) -> {
            //TODO: Deal with undoing and redoing settings changes
            mxUndoableEdit undoEdit = (mxUndoableEdit)evt.getProperty("edit");
            undoManager.undoableEditHappened(undoEdit);
        };
        
        graph.getModel().addListener(mxEvent.UNDO, undoHandler);
        graph.getView().addListener(mxEvent.UNDO, undoHandler);
        mxIEventListener undoHandler2;
        undoHandler2 = new mxIEventListener() {
            @Override
            public void invoke(Object source, mxEventObject evt) {
                List<mxUndoableChange> changes = ((mxUndoableEdit) evt.getProperty("edit")).getChanges();
                //Object cells[] = graph.getSelectionCellsForChanges(changes);
                //graph.setSelectionCells(cells);
                graph.setSelectionCells(new Object[]{});

                for(mxUndoableChange change : changes) {
                    if(change instanceof mxChildChange) {
                        mxChildChange childChange = ((mxChildChange)change);
                        if(childChange.getPrevious() == null && childChange.getChild() != null) {
                            if(childChange.getChild() instanceof mxProcessor) {
                                hgui.addProcessorToState((mxProcessor)childChange.getChild());
                            } else if(childChange.getChild() instanceof mxLink && ((mxLink)childChange.getChild()).getLink().isComplete()) {
                                hgui.addLinkToState((mxLink)childChange.getChild());
                            }
                        } else if(childChange.getPrevious() != null && childChange.getChild() != null && childChange.getPrevious() instanceof mxCell) {
                            if(childChange.getChild() instanceof mxProcessor) {
                                hgui.removeProcessorFromState((mxProcessor)childChange.getChild());
                            } else if(childChange.getChild() instanceof mxLink && ((mxLink)childChange.getChild()).getLink().isComplete()) {
                                hgui.removeLinkFromState((mxLink)childChange.getChild());
                            }
                        }
                    } /*else if(change instanceof mxTerminalChange) {
                        mxTerminalChange termChange = ((mxTerminalChange)change);
                        if(termChange.getTerminal() instanceof mxProcessor && termChange.getPrevious() instanceof mxProcessor && termChange.getCell() instanceof mxLink) {
                            System.out.println("HIT");
                            hgui.updateLinkTerminals((mxProcessor)termChange.getPrevious(), (mxProcessor)termChange.getTerminal(), (mxLink)termChange.getCell(), termChange.isSource());
                        }
                    }*/
                }
            }
        };
        undoManager.addListener(mxEvent.UNDO, undoHandler2);
        undoManager.addListener(mxEvent.REDO, undoHandler2);
        
        //Menu Bar Setup
        menuBar = new MenuBar(hgui);
        menuBar.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F10"), "none");
        this.setJMenuBar(menuBar);
        Log.debug("Menu bar created.");
        
        //JPanel Setup
        JPanel p = new JPanel(new BorderLayout());
        this.getContentPane().add(p);
        toolBar = new ToolBar(hgui);
        p.add(toolBar, BorderLayout.PAGE_START);
        
        Insets oldInsets = UIManager.getInsets("TabbedPane.contentBorderInsets");
        UIManager.put("TabbedPane.contentBorderInsets", new Insets(0, 0, 1, 0)); 
        propsTab = new PropertiesTab(hgui, state);
        UIManager.put("TabbedPane.contentBorderInsets", oldInsets); 
        Log.debug("Properties tab created.");
        
        logWindow = new LogWindow(hgui);
        Log.debug("Log window created.");
        
        splitPaneHoriz = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, propsTab, graphComponent);
        splitPaneVert = new JSplitPane(JSplitPane.VERTICAL_SPLIT, splitPaneHoriz, logWindow);
        splitPaneHoriz.setOneTouchExpandable(true);
        splitPaneHoriz.setDividerLocation(309);
        splitPaneHoriz.setDividerSize(6);
        splitPaneHoriz.setBorder(null);
        splitPaneVert.setOneTouchExpandable(true);
        splitPaneVert.setResizeWeight(1d);
        splitPaneVert.setBorder(null);
        
        logWindow.setVisible(false);
        splitPaneVert.setDividerSize(0);
        p.add(splitPaneVert, BorderLayout.CENTER);
        Log.debug("Splitpane setup.");
        
        statBar = new StatusBar();
        p.add(statBar, BorderLayout.PAGE_END);
        hgui.register(statBar);
        Log.debug("Status bar created.");
        
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.setVisible(true);
        statBar.resetText();
        
        this.addComponentListener(hgui);
        graphComponent.getGraphControl().addMouseMotionListener(hgui);
        graphComponent.getGraphControl().addMouseListener(hgui);
        graph.getSelectionModel().addListener(mxEvent.CHANGE, new mxIEventListener() {
            @Override
            public void invoke(Object o, mxEventObject eo) {
                if (o instanceof mxGraphSelectionModel) {
                    if(((mxGraphSelectionModel)o).isEmpty()) {
                        hgui.deselected();
                    } else {
                        hgui.selected(((mxGraphSelectionModel)o).getCells());
                    }
                }
            }
        });
        
        //Add listener to update network state when new cells added.
        graph.addListener(mxEvent.CELLS_ADDED, new mxIEventListener() {
            @Override
            public void invoke(Object o, mxEventObject eo) {
                Object cells[] = (Object[])eo.getProperty("cells");
                for(Object cell : cells) {
                    if(cell instanceof mxProcessor) {
                        if(((mxProcessor)cell).loading) {
                            ((mxProcessor)cell).loading = false;
                        } else {
                            hgui.addProcessorToState((mxProcessor)cell);
                        }
                    } else if(cell instanceof mxLink) {
                        mxLink link = (mxLink)cell;
                        
                        if(link.loading) {
                            link.loading = false;
                        } else {
                            if(link.isComplete()) {
                                hgui.addLinkToState((mxLink)cell);
                            }
                        }
                    }
                }
            }
        });
        
        //Add listener to update network state when new cells added.
        graph.addListener(mxEvent.CELLS_REMOVED, new mxIEventListener() {
            @Override
            public void invoke(Object o, mxEventObject eo) {
                Object cells[] = (Object[])eo.getProperty("cells");

                for(Object cell : cells) {
                    if(cell instanceof mxProcessor) {
                        hgui.removeProcessorFromState((mxProcessor)cell);
                    } else if(cell instanceof mxLink) {
                        mxLink link = (mxLink)cell;
                        if(link.isComplete()) {
                            hgui.removeLinkFromState((mxLink)cell);
                        }
                    }
                }
                
                //hgui.resetLayout(false);
            }
        });
        
        /*graph.addListener(mxEvent.CELLS_MOVED, new mxIEventListener() {
            @Override
            public void invoke(Object o, mxEventObject eo) {
                System.out.println("AAAAAAAAAAAAAAAAAA");
                hgui.resetLayout(false);
            }
        });*/
        
        
        mxIEventListener layoutResetListener = new mxIEventListener() {
            @Override
            public void invoke(Object o, mxEventObject eo) {
                if(eo.getName().equals(mxEvent.CELLS_MOVED)) {
                    Object cells[] = (Object[])eo.getProperty("cells");
                    for(Object cell: cells) {
                        if(cell instanceof mxProcessor) {
                            int x = (int) Math.round(((mxProcessor)cell).getGeometry().getX());
                            int y = (int) Math.round(((mxProcessor)cell).getGeometry().getY());
                            System.out.println( x+ " "+y);
                            ((mxProcessor)cell).getProcessor().setLocation(new Point(x,y));
                        }
                    }
                }
                
                hgui.resetLayout(false);
            }
        };
        
        graph.addListener(mxEvent.CELLS_MOVED, layoutResetListener);
        graph.addListener(mxEvent.CELLS_REMOVED, layoutResetListener);
        graph.addListener(mxEvent.CELLS_ADDED, layoutResetListener);
        graph.addListener(mxEvent.CELL_CONNECTED, layoutResetListener);
        graph.addListener(mxEvent.CELLS_FOLDED, layoutResetListener);
        graph.addListener(mxEvent.CELLS_ORDERED, layoutResetListener);
        graph.addListener(mxEvent.CELLS_RESIZED, layoutResetListener);
        graph.addListener(mxEvent.CELLS_TOGGLED, layoutResetListener);
        
        /*graph.addListener(mxEvent.CELL_CONNECTED, new mxIEventListener() {
            @Override
            public void invoke(Object o, mxEventObject eo) {
                mxLink mxl = (mxLink)eo.getProperty("edge");
                mxProcessor terminal = (mxProcessor)eo.getProperty("terminal");
                boolean source = (boolean)eo.getProperty("source");
                
                if(source && mxl.getLink().getSource() != terminal.getProcessor()) {
                    mxl.getLink().setSource(terminal.getProcessor(), this);
                } else if (!source && mxl.getLink().getTarget() != terminal.getProcessor()){
                    mxl.getLink().setTarget(terminal.getProcessor(), this);
                }
            }
        });*/
        
        graph.setStylesheet(new StyleSheet(graph));
        
        
        hkeyboard = new KeyboardHandler(p, hgui);
        
        
        /*//Default processor for new network
        graph.getModel().beginUpdate();
        Object proc = graph.insertVertex(graph.getDefaultParent(), null, "0", 1000, 1000, 75, 75, "running", false);
        graph.addSelectionCell(proc);
        graph.getModel().endUpdate();
        
        graphComponent.scrollCellToVisible(proc, true);
        //graphComponent.getParent().*/
        
        state.addListener(hgui, true);
        
        Log.debug("GUI Created.");
    }
    
    public void graphUndo() {
        undoManager.undo();
    }
    
    public void graphRedo() {
        undoManager.redo();
    }
    
    public GUIHandler getGUIHandler() {
        return hgui;
    }
    
    public void clearUndo() {
        undoManager.clear();
    }
}
