package dans.GUI;

import com.mxgraph.layout.mxGraphLayout;
import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxGraphModel;
import com.mxgraph.swing.handler.mxConnectPreview;
import com.mxgraph.swing.handler.mxPanningHandler;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.swing.util.mxCellOverlay;
import com.mxgraph.swing.util.mxGraphActions;
import com.mxgraph.util.mxConstants;
import com.mxgraph.util.mxPoint;
import com.mxgraph.view.mxCellState;
import com.mxgraph.view.mxGraph;
import dans.DANS;
import dans.algorithm.Algorithm;
import dans.network.BackEndMessage;
import dans.util.Log;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.awt.image.RGBImageFilter;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.TransferHandler;
import dans.network.Link;
import dans.network.NetworkObject;
import dans.network.NetworkObjectListener;
import dans.network.NetworkState;
import dans.network.NetworkStateListener;
import dans.network.Processor;
import dans.network.SettingsConstants;
import dans.util.AlgorithmLoader;
import dans.util.LogListener;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import javax.swing.JOptionPane;

/**
 *
 * @author Dan
 */
public class GUIHandler implements ComponentListener, MouseMotionListener, MouseListener, NetworkStateListener, NetworkObjectListener, LogListener {
    private MenuBar menuBar = null;
    private ToolBar toolBar = null;
    private PropertiesTab propsTab = null;
    private LogWindow logWindow = null;
    private mxGraphComponent graphComp = null;
    private StatusBar statBar = null;
    private GUI gui;
    
    private int lastLogWindowLoc = -1;
    private int lastPropsTabLoc = 261;
    private boolean gridEnabled = false;
    private int lastHeight = 0;
    private int lastWidth = 0;
    
    private mxCell lastCellOver = null;
    
    private mxLinkPreview preview = null;
    private mxCell previewCell = null;
    
    static GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices()[0];

    private int nodeWidth = 75;
    private int nodeHeight = 75;
    
    private Toolkit toolkit;
    private Cursor panning;
    private Cursor panningClosed;
    
    private NetworkState state;
    
    private mxGraphLayout bidirectionalLayout;
    
    
    public GUIHandler(GUI gui, NetworkState state) {
        this.gui = gui;
        toolkit = Toolkit.getDefaultToolkit();
        Image panImage = toolkit.getImage(this.getClass().getResource("/dans/imgs/panningCur.png"));
        Image panImageClosed = toolkit.getImage(this.getClass().getResource("/dans/imgs/panning2Cur.png"));
        Point hotSpot = new Point(0,0);
        panning = toolkit.createCustomCursor(panImage, hotSpot, "panning");
        panningClosed = toolkit.createCustomCursor(panImageClosed, hotSpot, "panningClosed");
        this.state = state;
    }
    
    public void register(Object obj) {
        if(obj instanceof MenuBar) {
            menuBar = (MenuBar)obj;
        } else if (obj instanceof ToolBar) {
            toolBar = (ToolBar)obj;
        } else if (obj instanceof PropertiesTab) {
            propsTab = (PropertiesTab)obj;
        } else if (obj instanceof LogWindow) {
            logWindow = (LogWindow)obj;
            Log.addListener(this, true);
        } else if (obj instanceof mxGraphComponent) {
            graphComp = (mxGraphComponent)obj;
            preview = new mxLinkPreview(graphComp, state);
            //resetLayout(false);
        } else if (obj instanceof StatusBar) {
            statBar = (StatusBar)obj;
        }
    }

    public void resetLayout(boolean move) {
        bidirectionalLayout = new NetworkLayout(graphComp.getGraph(), 30, gui.getContentPane().getSize().getWidth(), gui.getContentPane().getSize().getHeight(), move);
        bidirectionalLayout.execute(graphComp.getGraph().getDefaultParent());
    }
    
    public void showLogWindow() {
        showLogWindow(logWindow.isVisible());
    }
    
    public void showLogWindow(boolean b) {
        if(lastLogWindowLoc < 0 ) {
            lastLogWindowLoc = gui.getHeight() - 200;
        }

        if(b) {
            logWindow.setVisible(false);
            ((JSplitPane)logWindow.getParent()).setDividerSize(0);
            lastLogWindowLoc = ((JSplitPane)logWindow.getParent()).getDividerLocation();
            menuBar.setLogToggle(false);
            toolBar.setLogToggle(false);
        } else {
            logWindow.setVisible(true);
            ((JSplitPane)logWindow.getParent()).setDividerSize(6);
            ((JSplitPane)logWindow.getParent()).setDividerLocation(lastLogWindowLoc);
            menuBar.setLogToggle(true);
            toolBar.setLogToggle(true);
        }
    }

    public void showPropsTab() {
        showPropsTab(propsTab.isVisible());
    }
    
    public void showPropsTab(boolean b) {
        if(b) {
            propsTab.setVisible(false);
            ((JSplitPane)propsTab.getParent()).setDividerSize(0);
            lastPropsTabLoc = ((JSplitPane)propsTab.getParent()).getDividerLocation();
            menuBar.setPropsToggle(false);
            toolBar.setPropsToggle(false);
        } else {
            propsTab.setVisible(true);
            ((JSplitPane)propsTab.getParent()).setDividerSize(6);
            ((JSplitPane)propsTab.getParent()).setDividerLocation(lastPropsTabLoc);
            menuBar.setPropsToggle(true);
            toolBar.setPropsToggle(true);
        }
    }

    public void showGrid() {
        gridEnabled = !gridEnabled;
        showGrid(gridEnabled);
    }
    
    public void showGrid(boolean b) {
        graphComp.getGraph().setGridEnabled(b);
        graphComp.setGridVisible(b);
        menuBar.setGridToggle(b);
        toolBar.setGridToggle(b);
        graphComp.repaint();
        gridEnabled = b;
    }

    public void showToolsBar(boolean state) {
        toolBar.setVisible(state);
    }
    
    public void fullscreen() {
        fullscreen(!menuBar.getFullScreenToggle());
    }
    
    public void fullscreen(boolean b) {
        if(b) {
            lastHeight = gui.getHeight();
            lastWidth = gui.getWidth();
            gui.dispose();
            gui.setUndecorated(true);
            gui.pack();
            device.setFullScreenWindow(gui);
        } else {
            device.setFullScreenWindow(null);
            gui.dispose();
            gui.setUndecorated(false);
            gui.pack();
            gui.setSize(lastWidth, lastHeight);
            gui.setVisible(true);
        }
        
        menuBar.setFullScreenToggle(b);
    }

    public void play() {
        toolBar.lockUpForPlay();
        menuBar.lockUpForPlay();
        propsTab.lockUpForPlay();
        graphComp.setConnectable(false);
        graphComp.setDragEnabled(false);
        graphComp.getGraph().setCellsEditable(false);
        graphComp.getGraph().setCellsMovable(false);
        graphComp.getGraph().setCellsDisconnectable(false);
        graphComp.getGraph().setCellsCloneable(false);
        graphComp.getGraph().setConnectableEdges(false);
        graphComp.getGraph().setCellsBendable(false);
        graphComp.getGraph().setCellsDeletable(false);
        //TODO:Disable key shortcuts.
        
        graphComp.clearCellOverlays();
        editMode(ToolBar.EDIT_MODE_SELECT);
        
        Map cells = ((mxGraphModel)graphComp.getGraph().getModel()).getCells();
        for(Object id : cells.keySet()) {
            mxCell cell = (mxCell)cells.get(id);
            if(cell instanceof mxProcessor) {
                mxProcessor mxp = (mxProcessor)cell;
                graphComp.getGraph().setCellStyle("running", new Object[]{cell});
                mxp.setTopLabel("");
                mxp.setBottomLabel("");
            } else if(cell instanceof mxLink) {
                mxLink mxl = (mxLink)cell;
                graphComp.getGraph().setCellStyle("edgeoff", new Object[]{cell});
                mxl.setSourceLabel("");
                mxl.setTargetLabel("");
            }
        }
        
        graphComp.refresh();
        
        state.updateNetworkStat("Network State", SettingsConstants.STATE_RUNNING, this);
        gui.clearUndo();
    }
    
    public void pause() {
        toolBar.lockUpForPause();
        menuBar.lockUpForPause();
        state.updateNetworkStat("Network State", SettingsConstants.STATE_PAUSED, this);
        gui.clearUndo();
    }
    
    public void stop() {
        toolBar.lockUpForStop();
        menuBar.lockUpForStop();
    }
    
    public void newNetwork() {
        preview.stop(false);
        state.updateNetworkStat("Network State", SettingsConstants.STATE_CLEAR, this);
        toolBar.clearAlgorithms();
        statBar.setTick(0);
        statBar.setMsg("");
        propsTab.reset(state);
        mxLink.setLastID(0);
        mxProcessor.setLastID(0);
        gui.setTitle("DANS: New Network");
        lastCellOver = null;
        previewCell = null;
        graphComp.getGraph().removeCells(graphComp.getGraph().getChildCells(graphComp.getGraph().getDefaultParent(), true, true));
        zoomReset();
        graphComp.refresh();
        gui.clearUndo();
    }
    
    public void save() {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle("Save Network");
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setFileFilter(new SaveFilter());
        fc.setMultiSelectionEnabled(false);
        fc.setAcceptAllFileFilterUsed(false);
        int returnVal = fc.showSaveDialog(gui);
        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            int index = file.getName().lastIndexOf('.');
            
            if(index <= 0 || !file.getName().substring(index+1).equalsIgnoreCase("DANS")) {
                file = new File(file.toString() + ".dans");
            }
            
            gui.setTitle("DANS: " + file.getName());
            
            try {
                FileOutputStream fos = new FileOutputStream(file);
                ObjectOutputStream oos = new ObjectOutputStream(fos);
                oos.writeObject(state);
                fos.close();
                JOptionPane.showMessageDialog(gui, "Save successful.","Saved", JOptionPane.INFORMATION_MESSAGE);
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(gui, "Save failed!\n" + ex.getMessage(), "Save Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    public void load() {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle("Save Network");
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setFileFilter(new SaveFilter());
        fc.setMultiSelectionEnabled(false);
        fc.setAcceptAllFileFilterUsed(false);
        int returnVal = fc.showOpenDialog(gui);
        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            try {
                newNetwork();
                
		FileInputStream fis = new FileInputStream(fc.getSelectedFile());
		ObjectInputStream ois = new ObjectInputStream(fis);
		NetworkState newstate = (NetworkState)ois.readObject();
		ois.close();
                
                state.updateNetworkStat("Network State", SettingsConstants.STATE_LOAD, newstate);
                state.removeListener(this);
                state = newstate;
                state.addListener(this, true);
                
                gui.setTitle("DANS: " + fc.getSelectedFile().getName());
                
                int maxPID = -1;
                int maxLID = -1;
                
                graphComp.getGraph().getModel().beginUpdate();
                for(Processor p : newstate.getProcessors()) {
                    Point loc = p.getLocation();
                    mxProcessor proc = ((mxNetwork)graphComp.getGraph()).createProcessor(p, (double)loc.x, (double)loc.y);
                    proc.loading = true;
                    graphComp.getGraph().addCell(proc);
                    p.addListener(this, true);
                    
                    try {
                        int id = Integer.parseInt(p.getID());
                        if(id > maxPID) {
                            maxPID = id;
                        }
                    } catch(Exception e) {}
                }
                
                for(Link l : newstate.getLinks()) {
                    mxLink link = ((mxNetwork)graphComp.getGraph()).createLink(l);
                    link.loading = true;
                    Processor target = l.getTarget();
                    Processor source = l.getSource();
                    
                    if(target != null) {
                        mxProcessor mxp = this.findProcessor(target);
                        link.setTarget(mxp);
                    }
                    
                    if(source != null) {
                        mxProcessor mxp = this.findProcessor(source);
                        link.setSource(mxp);
                    } 
                    
                    graphComp.getGraph().addCell(link);
                    l.addListener(this, true);
                    
                    try {
                        int id = Integer.parseInt(l.getID());
                        if(id > maxLID) {
                            maxLID = id;
                        }
                    } catch(Exception e) {}
                }
                
                mxProcessor.setLastID(maxPID + 1);
                mxLink.setLastID(maxLID + 1);
                
                warningCheck();
                
                graphComp.getGraph().getModel().endUpdate();
                graphComp.refresh();
                
                for(String a: state.dumpAlgorithms()) {
                    Class algo = state.getAlgorithm(a);
                    if(a.equals(state.getDefaultAlgorthim())) {
                        toolBar.addAlgorithm(algo, true);
                    } else {
                        toolBar.addAlgorithm(algo, false);
                    }
                }
                propsTab.reset(state);
                
                gui.clearUndo();
                
                if(state.didAlgorthimsLoadFail()) {
                    JOptionPane.showMessageDialog(gui, "Some algorithms could not be loaded.\nThis can happen if you have moved the .class file to a new location since saving the network.", "Could not load all algorithms!", JOptionPane.WARNING_MESSAGE);
                }else {
                    JOptionPane.showMessageDialog(gui, "Load successful.","Loaded", JOptionPane.INFORMATION_MESSAGE);
                }
            } catch (IOException | ClassNotFoundException ex) {
                JOptionPane.showMessageDialog(gui, "Load failed!\n" + ex.getMessage(), "Load Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    public void reset() {
        state.updateNetworkStat("Network State", SettingsConstants.STATE_RESET, this);
        toolBar.unlockUpForReset();
        menuBar.unlockUpForReset();
        propsTab.unlockUpForReset();
        graphComp.setConnectable(true);
        graphComp.setDragEnabled(true);
        graphComp.getGraph().setCellsEditable(true);
        graphComp.getGraph().setCellsMovable(true);
        graphComp.getGraph().setCellsDisconnectable(true);
        graphComp.getGraph().setCellsCloneable(true);
        graphComp.getGraph().setConnectableEdges(true);
        graphComp.getGraph().setCellsBendable(false);
        graphComp.getGraph().setCellsDeletable(true);
        //TODO:Disable key shortcuts.
        
        graphComp.clearCellOverlays();
        editMode(ToolBar.EDIT_MODE_SELECT);
        
        propsTab.reset(state);
        statBar.setTick(0);
        
        graphComp.getGraph().setSelectionCell(null);
        
        Map cells = ((mxGraphModel)graphComp.getGraph().getModel()).getCells();
        for(Object id : cells.keySet()) {
            mxCell cell = (mxCell)cells.get(id);
            if(cell instanceof mxProcessor) {
                mxProcessor mxp = (mxProcessor)cell;
                graphComp.getGraph().setCellStyle("running", new Object[]{cell});
                mxp.setTopLabel("");
                mxp.setBottomLabel("");
            } else if(cell instanceof mxLink) {
                mxLink mxl = (mxLink)cell;
                graphComp.getGraph().setCellStyle("edgeoff", new Object[]{cell});
                mxl.setSourceLabel("");
                mxl.setTargetLabel("");
            }
        }
        
        warningCheck();
        graphComp.refresh();
        gui.clearUndo();
    }
    
    public void exit() {
        //TODO: Ask user before exiting.
        //TODO: Ask user to save before exiting.
        System.exit(0);
    }
    
    public void print() {
        PrinterJob job = PrinterJob.getPrinterJob();
        PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
        PageFormat pf = job.pageDialog(aset);
        job.setPrintable(new PrintOut(graphComp), pf);
        if (job.printDialog()) {
            try {job.print();}
            catch (PrinterException exc) {
                //TODO: Output some error here.
            }
        }  
    }
    
    public void zoomIn() {
        graphComp.zoomIn();
    }
    
    public void zoomOut() {
        graphComp.zoomOut();
    }
    
    public void zoomReset() {
        graphComp.zoomActual();
    }
    
    public void undo() {
        gui.graphUndo();
    }
    
    public void redo() {
        gui.graphRedo();
    }
    
    public void exportImage() {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle("Export Image of Network Graph");
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setFileFilter(new PNGFilter());
        fc.setMultiSelectionEnabled(false);
        fc.setAcceptAllFileFilterUsed(false);
        int returnVal = fc.showSaveDialog(gui);
        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            int index = file.getName().lastIndexOf('.');
            
            if(index <= 0 || !file.getName().substring(index+1).equalsIgnoreCase("PNG")) {
                file = new File(file.toString() + ".png");
            }

            BufferedImage image = new BufferedImage(graphComp.getWidth(), graphComp.getHeight(), BufferedImage.TYPE_INT_RGB);
            Graphics g = image.createGraphics();
            g.fillRect(0, 0, image.getWidth(), image.getHeight());
            graphComp.print(g);
            
            //Make background transparent.
            //Based on code from http://stackoverflow.com/questions/665406/how-to-make-a-color-transparent-in-a-bufferedimage-and-save-as-png
            ImageFilter filter = new RGBImageFilter() {
                public int markerRGB = graphComp.getBackground().getRGB() | 0xFF000000;
                @Override
    		public final int filterRGB(int x, int y, int rgb) {
    			if ((rgb | 0xFF000000) == markerRGB) {
    				return 0x00FFFFFF & rgb;
    			} else {
    				return rgb;
    			}
    		}
            };
            ImageProducer ip = new FilteredImageSource(image.getSource(), filter);
            Image im = Toolkit.getDefaultToolkit().createImage(ip);
            image = new BufferedImage(im.getWidth(null), im.getHeight(null), BufferedImage.TYPE_INT_ARGB);
            Graphics2D g2 = image.createGraphics();
            g2.drawImage(im, 0, 0, null);
            g2.dispose();
            image = image.getSubimage(2, 2, image.getWidth()-4, image.getHeight()-4); //crop out borders
            //End code to make transparent.
            
            
            try {
                ImageIO.write(image, "PNG", file);
            } catch (IOException ex) {
                //TODO: Show an error or somthing here.
            }
        } else {
            //TODO: Show an error or somthing here.
        }
    }

    public void cut(ActionEvent e) {
        selectOnlyProcessors();
        TransferHandler.getCutAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }
    
    public void paste(ActionEvent e) {
        TransferHandler.getPasteAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
        //((mxNetwork)graphComp.getGraph()).resolveDuplicateProcessors(state);
        // graphComp.refresh();
    }
    
    public void selectOnlyProcessors() {
        ArrayList<Object> select = new ArrayList<>();
        for(Object cell : graphComp.getGraph().getSelectionCells()) {
            if(cell instanceof mxProcessor) {
                select.add(cell);
            }
        }
        graphComp.getGraph().setSelectionCells(select.toArray());
    }
    
    public void copy(ActionEvent e) {
        selectOnlyProcessors();
        TransferHandler.getCopyAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }
    
    public void delete(ActionEvent e) {
        mxGraphActions.getDeleteAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }
    
    public void selectAll(ActionEvent e) {
        mxGraphActions.getSelectAllAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }
    
    public void selectNone(ActionEvent e) {
        mxGraphActions.getSelectNoneAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }
    
    
    private void selectVerts(ActionEvent e) {
        mxGraphActions.getSelectVerticesAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }

    private void selectEdges(ActionEvent e) {
        mxGraphActions.getSelectEdgesAction().actionPerformed(new ActionEvent(graphComp, e.getID(), e.getActionCommand()));
    }

    @Override
    public void componentResized(ComponentEvent e) {
        statBar.resetText();
    }

    @Override
    public void componentMoved(ComponentEvent e) {
       
    }

    @Override
    public void componentShown(ComponentEvent e) {
        
    }

    @Override
    public void componentHidden(ComponentEvent e) {
        
    }

    void showStatBar(boolean state) {
        statBar.setVisible(state);
        statBar.resetText();
    }

    void showMenuBar(boolean state) {
        menuBar.setVisible(state);
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        mxPoint point = graphComp.getPointForEvent(e);
        statBar.setLoc((int)point.getX(),(int)point.getY());
        
        if(toolBar.getEditMode() == ToolBar.EDIT_MODE_PAN && graphComp.getPanningHandler().isActive()) {
            graphComp.getGraphControl().setCursor(panningClosed); 
        }
    }
    
    private void highlightNode(mxCell cell) {
        if(toolBar.getEditMode() == ToolBar.EDIT_MODE_LINK || toolBar.getEditMode() == ToolBar.EDIT_MODE_DELETE) {            
            if(cell != null && cell != lastCellOver) {
                if(cell.isVertex()) {
                    //TODO: Account for root nodes
                    cell.setStyle("runninghighlight");
                } else if (cell.isEdge() && toolBar.getEditMode() == ToolBar.EDIT_MODE_DELETE) {
                    cell.setStyle("edgeonhighlight");
                }

                if(lastCellOver != null) {
                    if(lastCellOver.isVertex()) {
                        //TODO: Account for root nodes
                        lastCellOver.setStyle("running");
                    } else if(lastCellOver.isEdge()) {
                        lastCellOver.setStyle("edgeoff");
                    }
                }
                graphComp.refresh();

                lastCellOver = cell;
            } else if(lastCellOver != null && cell != lastCellOver) {
                if(lastCellOver.isVertex()) {
                    //TODO: Account for root nodes
                    lastCellOver.setStyle("running");
                } else if(lastCellOver.isEdge()) {
                    lastCellOver.setStyle("edgeoff");
                }
                graphComp.refresh();

                lastCellOver = null;
            }
            
            /*if(cell != null && (cell.isVertex() || (cell.isEdge() && toolBar.getEditMode() == ToolBar.EDIT_MODE_DELETE))) {
                setCursor(gui.getRootPane(), new Cursor(Cursor.HAND_CURSOR));
            }*/
        }
    } 
    
    @Override
    public void mouseMoved(MouseEvent e) {
        mxCell cell = (mxCell)graphComp.getCellAt(e.getX(),e.getY());
        if(preview.isActive() && (cell == null || !cell.isVertex() || cell == previewCell)) {
            preview.update(e, null, e.getX(), e.getY());
        } else if(preview.isActive() && cell != null) {
            preview.update(e, new mxCellState(graphComp.getGraph().getView(), cell, null), e.getX(), e.getY());
        }
        
        if(cell != null && (toolBar.getEditMode() == ToolBar.EDIT_MODE_LINK || toolBar.getEditMode() == ToolBar.EDIT_MODE_DELETE)) {
            if(cell.isVertex()) {
                graphComp.getGraphControl().setCursor(new Cursor(Cursor.HAND_CURSOR));
            } else if(cell.isEdge() && toolBar.getEditMode() == ToolBar.EDIT_MODE_DELETE) {
                graphComp.getGraphControl().setCursor(new Cursor(Cursor.HAND_CURSOR));
            }
        } else if (toolBar.getEditMode() == ToolBar.EDIT_MODE_PAN) {
            if(!graphComp.getPanningHandler().isActive()) {
                graphComp.getGraphControl().setCursor(panning); 
            }
        }
        
        mxPoint point = graphComp.getPointForEvent(e);
        statBar.setLoc((int)point.getX(),(int)point.getY());
    }
    
    @Override
    public void mouseClicked(MouseEvent e) {
    }

    @Override
    public void mousePressed(MouseEvent e) {
        if (toolBar.getEditMode() == ToolBar.EDIT_MODE_PAN) {
            graphComp.getGraphControl().setCursor(panningClosed); 
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        mxCell cell = (mxCell)graphComp.getCellAt(e.getX(),e.getY());
        boolean madeCell = false;
        
        if(toolBar.getEditMode() == ToolBar.EDIT_MODE_PROC) {
            if(cell == null) {
                graphComp.getGraph().getModel().beginUpdate();
                mxPoint point = graphComp.getPointForEvent(e);

                Processor newP = new Processor("" + mxProcessor.getLastID());
                newP.setDefaultSettings(state);
                newP.setDefaultStats(state);
                //state.addProcessor(newP, this);
                
                mxProcessor proc = ((mxNetwork)graphComp.getGraph()).insertProcessor(newP, point.getX() - (mxProcessor.DEFAULT_SIZE/2), point.getY() - (mxProcessor.DEFAULT_SIZE/2));
                proc.getProcessor().setLocation(new Point((int)point.getX() - (int)(mxProcessor.DEFAULT_SIZE/2), (int)point.getY() - (int)(mxProcessor.DEFAULT_SIZE/2)));
                
                /*if(!newP.hasAlgorithmSet()) {
                    graphComp.setCellWarning(proc, "Algorithm not loaded");
                }*/
                
                graphComp.getGraph().addSelectionCell(proc);
                graphComp.getGraph().getModel().endUpdate();
                madeCell = true;
            }
        } else if(toolBar.getEditMode() == ToolBar.EDIT_MODE_LINK) {
            if(cell instanceof mxProcessor) {
                if(preview.isActive()) {
                    if(cell == previewCell || !cell.isVertex()) {
                        preview.stop(false);
                    } else {
                        graphComp.getGraph().getModel().beginUpdate();
                        preview.stop(true);
                        graphComp.getGraph().getModel().endUpdate();
                    }

                    previewCell = null;
                }

                if(cell != previewCell && cell.isVertex()) {
                    preview.start(null, new mxCellState(graphComp.getGraph().getView(), cell, null), "edgeoff");
                    previewCell = cell;
                }
            } else {
                preview.stop(false);
                previewCell = null;
            }
        } else if (toolBar.getEditMode() == ToolBar.EDIT_MODE_PAN) {
            graphComp.getGraphControl().setCursor(panning); 
        } else if(toolBar.getEditMode() == ToolBar.EDIT_MODE_DELETE && cell != null) {
            if(cell instanceof mxProcessor || cell instanceof mxLink) {
                graphComp.getGraph().getModel().beginUpdate();
                graphComp.getGraph().removeCells(new Object[]{cell});
                graphComp.getGraph().getModel().endUpdate();
            }
        }
        
        /*if(cell != null) {
            resetLayout(false);
        }*/
    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(MouseEvent e) {
        statBar.setLoc(0,0);
    }

    public void selected(Object[] cells) {
        if(cells.length == 1) {
            mxCell cell = (mxCell)cells[0];
            propsTab.displaySettingsFor(cell);
            propsTab.displayStatsFor(cell);
            if(cell instanceof mxProcessor) {
                propsTab.displayAlgorithmsFor((mxProcessor) cell);
            } else {
                propsTab.displayAlgorithmsFor(null);
            }
        } else {
            propsTab.displaySettingsFor(null);
            propsTab.displayStatsFor(null);
            propsTab.displayAlgorithmsFor(null);
        }
    }

    public void editMode(int editMode) {
        toolBar.setEditMode(editMode);
        
        if(preview.isActive()) {
            preview.stop(false);
        }
        previewCell = null;
        
        //graphComp.getGraph().setSelectionCell(null);
        if(editMode == ToolBar.EDIT_MODE_SELECT) {
            graphComp.getGraph().setCellsSelectable(true);
            graphComp.setDragEnabled(true);
            graphComp.getGraph().setCellsMovable(true);
            graphComp.setConnectable(false);
        } else if(editMode == ToolBar.EDIT_MODE_LINK) {
            graphComp.getGraph().setCellsSelectable(true);
            graphComp.getGraph().setCellsMovable(false);
            graphComp.setConnectable(false);
            graphComp.setDragEnabled(false);
        } else {
            graphComp.getGraph().setCellsSelectable(true);
            graphComp.getGraph().setCellsMovable(false);
            graphComp.setConnectable(false);
            graphComp.setDragEnabled(false);
        }
        
    }
    
    public void setLastProcessorID(int id) {
        mxProcessor.setLastID(id);
    }
    
    public void setLastLinkID(int id) {
        mxLink.setLastID(id);
    }

    public boolean isPanningEvent(MouseEvent e) {
        return toolBar != null && toolBar.getEditMode() == ToolBar.EDIT_MODE_PAN && e.getButton() ==  MouseEvent.BUTTON1;
    }

    public void setCellStyle(mxCell cell, String style) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    public void setCellText(mxCell cell, String text) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    public void setStatusBarMessage(String msg) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void logUpdate(String msg) {
        logWindow.addToLog(msg);
    }

    public void clearLog() {
        logWindow.clearLog();
    }

    public void networkSettingsUpdate(String rowName, Object value) {
        if(rowName.equals("loggingLevel")) {
            Log.setLogLevel(value.toString());
        }
        
        state.updateNetworkSetting(rowName, value, this);
    }

    @Override
    public void processorChange(Processor p, int eventID, Object caller) {
        
    }

    @Override
    public void linkChange(Link p, int eventID, Object caller) {
       
    }

    @Override
    public void settingsChange(String name, Object value, int eventID, Object caller) {
        if(eventID == NetworkState.EVENT_SETTINGS_UPDATE) {
            propsTab.setSettingsValue(name, value);
        }
    }

    @Override
    public void statsChange(String name, Object value, int eventID, Object caller) {
        if(eventID == NetworkState.EVENT_STAT_UPDATE) {
            propsTab.setStatsValue(name, value);
            
            if(name.equals("Network State") && value != null && value.equals(SettingsConstants.STATE_STOPPED)) {
                stop();
            }
        }
    }

    @Override
    public void tickChange(int tickValue, int eventID, Object caller) {
        statBar.setTick(tickValue);
        propsTab.setStatsValue("Tick", tickValue);
        
       Map cells = ((mxGraphModel)graphComp.getGraph().getModel()).getCells();
        for(Object id : cells.keySet()) {
            mxCell cell = (mxCell)cells.get(id);
            if(cell instanceof mxLink) {
                Link l = ((mxLink)cell).getLink();                   
                ((mxLink)cell).setTargetLabel("");
                
                if(!l.getStatus().equals(NetworkObject.STATUS_ERROR) &&  !l.getStatus().equals(NetworkObject.STATUS_FAILED)) {
                    if(l.getQueue().getLastTransfer() < tickValue) {
                        graphComp.getGraph().setCellStyle("edgeoff", new Object[]{cell});
                    }
                }
            }
        }
        
        graphComp.refresh();
    }

    public void addProcessorToState(mxProcessor mxp) {
        Map cells = ((mxGraphModel)graphComp.getGraph().getModel()).getCells();
        
        for(Object id : cells.keySet()) {
            mxCell cell = (mxCell)cells.get(id);
            if(cell instanceof mxProcessor) {
                if(cell != mxp && cell.getValue().equals(mxp.getValue())) {
                    mxp.setProcessor(new Processor(mxProcessor.getLastID() + "", mxp.getProcessor()));
                    mxProcessor.clearBackup(cell.getValue().toString());
                    break;
                }
            }
        }
        
        if(!mxp.getProcessor().hasAlgorithmSet()) {
            graphComp.setCellWarning(mxp, "Algorithm not loaded");
        }
        
        state.addProcessor(mxp.getProcessor(), this);
        mxp.getProcessor().addListener(this, true);
    }

    public void addLinkToState(mxLink mxl) {
        /*Map cells = ((mxGraphModel)graphComp.getGraph().getModel()).getCells();
        for(Object id : cells.keySet()) {
            mxCell cell = (mxCell)cells.get(id);
            if(cell instanceof mxLink) {
                if(cell != mxl && ((mxLink)cell).getLink().getID().equals(mxl.getLink().getID())) {
                    mxl.setLink(new Link(mxLink.getLastID()+"", mxl.getLink()));
                    mxLink.clearBackup(mxl.getLink().getID());
                    break;
                }
            }
        }*/
        mxLink.clearBackup(mxl.getLink().getID());
        
        state.addLink(mxl.getLink(), this);
        mxl.getLink().addListener(this, true);
    }

    public void removeProcessorFromState(mxProcessor mxp) {
        if(state.processorIDUsed(mxp.getProcessor().getID())) {
            mxp.getProcessor().removeListener(this);
            state.removeProcessor(mxp.getProcessor(), this);
        } else {
            Log.debug("Remove request for processor id, " + mxp.getProcessor().getID()+ ", that is not in network state.");
        }
    }

    public void removeLinkFromState(mxLink mxl) {
        if(state.linkIDUsed(mxl.getLink().getID())) {
            mxl.getLink().removeListener(this);
            state.removeLink(mxl.getLink(), this);
        } else{
            Log.debug("Remove request for link id, " + mxl.getLink().getID()+ ", that is not in network state.");
        }
    }

    public void deselected() {
        propsTab.displaySettingsFor(null);
        propsTab.displayStatsFor(null);
        propsTab.displayAlgorithmsFor(null);
    }

    public void processorSettingsUpdate(String rowName, Object value, Processor p) {
        if(rowName.equals("ID")) {
            p.setID(value.toString(), this);
            graphComp.refresh();
        } else {
            p.setSetting(rowName, value, this);
        } 
    }

   /* void updateLinkTerminals(mxProcessor previous, mxProcessor terminal, mxLink link, boolean source) {
        if(source) {
            
        }
    }*/

    public void linkSettingsUpdate(String rowName, Object value, Link l) {
        if(rowName.equals("ID")) {
            l.setID(value.toString(), this);
            graphComp.refresh();
        } else {
            l.setSetting(rowName, value, this);
        }
    }

    public void loadAlgorithm() {
        JFileChooser fc = new JFileChooser();
        fc.setCurrentDirectory(new File(System.getProperty("user.dir")));
        fc.setDialogTitle("Load Algorithm Class");
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setFileFilter(new ClassFilter());
        fc.setMultiSelectionEnabled(false);
        fc.setAcceptAllFileFilterUsed(false);
        int returnVal = fc.showOpenDialog(gui);
        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            Class algorithm = AlgorithmLoader.loadAlgorithm(file);
            if(algorithm != null) {
                if(state.addAlgorithm(algorithm, file)) {
                    if(state.dumpAlgorithms().length == 1) {
                        toolBar.addAlgorithm(algorithm, true);
                    } else {
                        toolBar.addAlgorithm(algorithm, false);
                    }
                    
                    JOptionPane.showMessageDialog(gui, "Algorithm " + algorithm.getSimpleName()  + " loaded successfully.\nYou can now select it as the default option in the default dropdown or manually apply it to a processor.", "Algorithm Loaded Successfully", JOptionPane.INFORMATION_MESSAGE);
                } else {
                    JOptionPane.showMessageDialog(gui, "Algorithm already loaded.\nEnsure that the Algorithm class has a unique name and is not allready loaded.", "Algorithm Load Error", JOptionPane.ERROR_MESSAGE);
                }
            } else {
                JOptionPane.showMessageDialog(gui, "Failed to load Algorithm class.\nEnsure that the class extends " + Algorithm.class.getName() + ", contains no errors and is a vaild java class file.\nSee log for more details on why the class could not be laoded.", "Algorithm Load Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void setDefaultAlgorithm(String algorithm) {
        if(!algorithm.equals(state.getNetworkSetting("defaultAlgorithm")) && !(state.getNetworkSetting("defaultAlgorithm").equals("") && algorithm.equals("<None>"))) {
            if(!algorithm.equals("<None>")) {
                state.setDefaultAlgorithm(algorithm);
            } else {
                state.setDefaultAlgorithm("");
            }
            
            warningCheck();
        }
    }

    public void warningCheck() {
        Object mxps[] = graphComp.getGraph().getChildVertices(graphComp.getGraph().getDefaultParent());
        for(Object mxp : mxps) {
            warningCheck((mxProcessor)mxp);
        }
    }
    
    public void warningCheck(mxProcessor mxp) {
        graphComp.removeCellOverlays(mxp);
        if(!((mxProcessor)mxp).getProcessor().hasAlgorithmSet()) {
            graphComp.setCellWarning(mxp, "Algorithm not loaded");
        }
    }

    public String[] getAlgorithms() {
        return state.dumpAlgorithms();
    }

    private mxProcessor findProcessor(Processor p) {
        return mxProcessor.findProcessor(p, (mxGraphModel)graphComp.getGraph().getModel());
    }
    
    private mxLink findLink(Link l) {
        return mxLink.findLink(l, (mxGraphModel)graphComp.getGraph().getModel());
    }
    
    @Override
    public void networkObjectChange(NetworkObject object, Object oldvalue, int eventID, Object caller) {
        if(object instanceof Processor) {
            Processor p = (Processor)object;
            
            
            if(eventID == NetworkObject.EVENT_STATUS_UPDATE) {
                if(p.getStatus().equals(NetworkObject.STATUS_ERROR)) {
                    mxProcessor mxp = findProcessor(p);
                    graphComp.getGraph().setCellStyle("stopped", new Object[]{mxp});
                    //TODO: add error overlay
                } else if(p.getStatus().equals(NetworkObject.STATUS_TERMINATED)) {
                    mxProcessor mxp = findProcessor(p);
                    graphComp.getGraph().setCellStyle("stopped", new Object[]{mxp});
                } else if(p.getStatus().equals(NetworkObject.STATUS_FAILED)) {
                    mxProcessor mxp = findProcessor(p);
                    graphComp.getGraph().setCellStyle("failed", new Object[]{mxp});
                }
            } else if(eventID == Processor.EVENT_PART_FAIL) {
                mxProcessor mxp = findProcessor(p);
                graphComp.getGraph().setCellStyle("failed", new Object[]{mxp});
            } else if(eventID == Processor.EVENT_DISPLAY) {
                if(!p.getStatus().equals(NetworkObject.STATUS_NOTSTARTED)) {
                    mxProcessor mxp = findProcessor(p);
                    String text = (String)((Object[])oldvalue)[0];
                    int pos = (int)((Object[])oldvalue)[1];
                    if(pos == 0) {
                        Log.debug(text, "P:" + mxp.getValue() + "#" + ((Algorithm)caller).getPort());
                        mxp.setTopLabel(text);
                    } else if(pos == 1) {
                        Log.debug(text, "P:" + mxp.getValue() + "#" + ((Algorithm)caller).getPort());
                        mxp.setBottomLabel(text);
                    } else {
                        Log.info(text, "P:" + mxp.getValue() + "#" + ((Algorithm)caller).getPort());
                    }
                    graphComp.refresh();
                }
            }
        } else if(object instanceof Link) {
            Link l = (Link)object;
            mxLink mxl = null;
            
            if(!l.getStatus().equals(NetworkObject.STATUS_FAILED) && !l.getStatus().equals(NetworkObject.STATUS_ERROR)) {
                if(eventID == Link.EVENT_MESSAGE_SENT) {
                    mxl = findLink(l);
                    BackEndMessage bm = (BackEndMessage)oldvalue;
                    graphComp.getGraph().setCellStyle("edgeon", new Object[]{mxl});
                    mxl.setSourceLabel(bm.msg);
                    graphComp.refresh();
                } else if(eventID == Link.EVENT_MESSAGE_TRANSFERRED) {
                    mxl = findLink(l);
                    BackEndMessage bm = (BackEndMessage)oldvalue;
                    mxl.setTargetLabel(bm.msg);
                    if(l.getQueue().getSize() <= 0) {
                        mxl.setSourceLabel("");
                    }
                    graphComp.refresh();
                }
            } else if(eventID == Link.EVENT_STATUS_UPDATE) {
                mxl = findLink(l);
                graphComp.getGraph().setCellStyle("failededge", new Object[]{mxl});
                mxl.setSourceLabel("");
                mxl.setTargetLabel("");
                graphComp.refresh();
            }
        }
        
    }
    
    public static class UndoAction extends AbstractAction{
        GUIHandler hgui;
        
        public UndoAction(GUIHandler hgui) {
            this.hgui = hgui;
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            hgui.undo();
        }
    };
    
    public static class RedoAction extends AbstractAction{
        GUIHandler hgui;
        
        public RedoAction(GUIHandler hgui) {
            this.hgui = hgui;
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            hgui.redo();
        }
    };
    
    public static class EditModeAction extends AbstractAction{
        GUIHandler hgui;
        int mode;
        
        public EditModeAction(GUIHandler hgui, int mode) {
            this.hgui = hgui;
            this.mode = mode;
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            hgui.editMode(mode);
        }
    };
    
    public static class FullScreenAction extends AbstractAction{
        GUIHandler hgui;
        
        public FullScreenAction(GUIHandler hgui) {
            this.hgui = hgui;
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            hgui.fullscreen();
        }
    };
    
    public static class EditingAction extends AbstractAction{
        public static final int EDIT_MODE_COPY = 1;
        public static final int EDIT_MODE_CUT = 2;
        public static final int EDIT_MODE_PASTE = 3;
        public static final int EDIT_MODE_DELETE = 4;
        public static final int EDIT_MODE_SELECT_ALL = 5;
        public static final int EDIT_MODE_SELECT_NONE = 6;
        public static final int EDIT_MODE_SELECT_VERT = 7;
        public static final int EDIT_MODE_SELECT_EDGE = 8;
        
        GUIHandler hgui;
        int mode;
        
        public EditingAction(GUIHandler hgui, int mode) {
            this.hgui = hgui;
            this.mode = mode;
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            switch(mode) {
                case EDIT_MODE_COPY:
                    hgui.copy(e);
                    break;
                case EDIT_MODE_CUT:
                    hgui.cut(e);
                    break;
                case EDIT_MODE_PASTE:
                    hgui.paste(e);
                    break;
                case EDIT_MODE_DELETE:
                    hgui.delete(e);
                    break;
                case EDIT_MODE_SELECT_ALL:
                    hgui.selectAll(e);
                    break;
                case EDIT_MODE_SELECT_NONE:
                    hgui.selectNone(e);
                    break;
                case EDIT_MODE_SELECT_VERT:
                    hgui.selectVerts(e);
                    break;
                case EDIT_MODE_SELECT_EDGE:
                    hgui.selectEdges(e);
                    break;
                default:
                    break;
            }
        }
    };
    
    public static class ZoomAction extends AbstractAction{
        public static final int ZOOM_MODE_OUT = 1;
        public static final int ZOOM_MODE_IN = 2;
        public static final int ZOOM_MODE_RESET = 3;
        
        GUIHandler hgui;
        int mode;
        
        public ZoomAction(GUIHandler hgui, int mode) {
            this.hgui = hgui;
            this.mode = mode;
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            switch(mode) {
                case ZOOM_MODE_OUT:
                    hgui.zoomOut();
                    break;
                case ZOOM_MODE_IN:
                    hgui.zoomIn();
                    break;
                case ZOOM_MODE_RESET:
                    hgui.zoomReset();
                    break;
                default:
                    break;
            }
        }
    };
}
