package dans.GUI;

import static dans.GUI.NetworkSettingsTableModel.rowNames;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import javax.swing.table.AbstractTableModel;
import dans.network.Link;
import dans.network.NetworkObject;
import dans.network.NetworkObjectListener;
import dans.network.Processor;

/**
 *
 * @author Dan
 * @todo Refactor this and ProcessorSettingsTableModel into one class.
 * @todo Add tool tips.
 */
public class LinkSettingsTableModel extends AbstractTableModel implements NetworkObjectListener  {
    Object[][] rowData;
    public static final String[] columnNames = {"Setting Name", "Value"};
    public static final String rowNames[] = {
        "ID",
        "Delay",
        "Bandwidth",
        "",
        "Speed Change Method",
        "Change Seed",
        "Change Min",
        "Change Max",
        "Normal Change Mean",
        "Normal Change STDV",
        "Uniform Change Min",
        "Uniform Change Max",
        "Constant Change Amount",
        "",
        "Failure Rate",
        "Packet Loss Rate",
        "Byte Error Rate"
    };
    private GUIHandler hgui;
    private Link l;

    public static final Set<String> typeDouble = new HashSet<>(Arrays.asList(
        new String[] {"Failure Rate", "Packet Loss Rate", "Byte Error Rate", "Bandwidth"}
    ));
    
    public static final Set<String> typeNeg = new HashSet<>(Arrays.asList(
        new String[] {"Normal Change Mean", "Uniform Change Min", "Uniform Change Max", "Constant Change Amount"}
    ));
    
    public static final Set<String> typeMethod = new HashSet<>(Arrays.asList(
        new String[] {"Speed Change Method"}
    ));
    
    public LinkSettingsTableModel(GUIHandler hgui, Link link) {
        rowData = link.dumpSettings(LinkSettingsTableModel.rowNames);
        this.hgui = hgui;
        l = link;
        l.addListener(this, true);
    }
    
    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    @Override
    public String getColumnName(int column) {
        return columnNames[column];
    }

    @Override
    public int getRowCount() {
        return rowData.length;
    }

    @Override
    public Object getValueAt(int row, int column) {
        return rowData[row][column];
    }

    @Override
    public Class getColumnClass(int column) {
        if(column == 0) {
            return String.class;
        } else {
            return Integer.class;
        }
    }

    @Override
    public void setValueAt(Object value, int row, int column) {        
        if(!rowData[row][column].equals(value)) {
            rowData[row][column] = value;
            fireTableCellUpdated(row, column);
            
            if(value instanceof String) {
                value = ((String)value).replaceAll(",", "");
            }
            
            hgui.linkSettingsUpdate(rowNames[row], value, l);
        }
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        //TODO: Fix bug with changing link ID.
        return column == 1 && !LinkSettingsTableModel.rowNames[row].equals("") && !LinkSettingsTableModel.rowNames[row].equals("ID");
    } 

    @Override
    public void networkObjectChange(NetworkObject object, Object oldvalue, int eventID, Object caller) {
        if(caller != hgui) {
            if(eventID == NetworkObject.EVENT_SETTINGS_UPDATE) {
                String name = (String)((Object[])oldvalue)[0];
                int row = Arrays.asList(rowNames).indexOf(name);
                if(row >= 0) {
                    rowData[row][1] = object.getSetting(name);
                    fireTableCellUpdated(row, 1);
                }
            } else if(eventID == NetworkObject.EVENT_ID_UPDATE) {
                int row = Arrays.asList(rowNames).indexOf("ID");
                if(row >= 0) {
                    rowData[row][1] = object.getSetting(object.getID());
                    fireTableCellUpdated(row, 1);
                }
            }
        }
    }
    
    public void removeListener() {
        l.removeListener(this);
    }
}
