package dans.GUI;


import com.mxgraph.layout.mxGraphLayout;
import com.mxgraph.model.mxCell;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import com.mxgraph.model.mxCellPath;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxGraphModel;
import com.mxgraph.model.mxICell;
import com.mxgraph.model.mxIGraphModel;
import com.mxgraph.util.mxPoint;
import com.mxgraph.util.mxRectangle;
import com.mxgraph.view.mxCellState;
import com.mxgraph.view.mxGraph;
import com.mxgraph.view.mxGraphView;
import java.awt.geom.Rectangle2D;

//This calass is based on an existing layout from JGraph.
//Parts of the source code for this class are from mxParallelEdgeLayout.java in JGraphX.
public class NetworkLayout extends mxGraphLayout {

private int spacing;
private double frameheight, framewidth;
private boolean move = true;


public NetworkLayout(mxGraph graph, int spacing, double framewidth, double frameheight, boolean move)
{
super(graph);
this.spacing = spacing;
this.frameheight = frameheight;
this.framewidth = framewidth;
this.move = move;
}


public void execute(Object parent)
{
Map<String, List<Object>> lookup = findParallels(parent);


graph.getModel().beginUpdate();
resetEdges();
try
{
    if(move) {
        Object[] vertices = graph.getChildVertices(parent);
        Rectangle2D.Double bound = new Rectangle2D.Double(Double.MAX_VALUE,Double.MAX_VALUE,0.0,0.0);
        for(int i = 0; i < vertices.length; i++) {
            mxRectangle rect = graph.getCellBounds(vertices[i], false, true);
            if(rect.getX() < bound.getX()) {
                bound.x = rect.getX();
            }

            if(rect.getY() < bound.getY()) {
                bound.y = rect.getY();
            }
        }

        for(int i = 0; i < vertices.length; i++) {
            mxRectangle rect = graph.getCellBounds(vertices[i], false, true);
            if(rect.getY() + rect.getHeight() > bound.getHeight() + bound.getY()) {
                bound.height = rect.getY() + rect.getHeight() - bound.getY();
            }

            if(rect.getX() + rect.getWidth() > bound.getWidth() + bound.getX()) {
                bound.width = rect.getX() + rect.getWidth() - bound.getX();
            }
        }
        
        if(bound.getHeight() > frameheight) {
            bound.y = (bound.y) * -1;
        } else {
            bound.y = ((frameheight - bound.getHeight()) / 2.0 - 0) - bound.y;
        }

        if(bound.getWidth() > framewidth - 37.5) {
            bound.x = (bound.x) * -1;
        } else {
            bound.x = ((framewidth - bound.getWidth()) / 2.0 - 100) - bound.x;
        }
        
        graph.moveCells(vertices, bound.x, bound.y);    
    }
    
    
Iterator<List<Object>> it = lookup.values().iterator();
while (it.hasNext())
{
List<Object> parallels = it.next();
if (parallels.size() > 1)
{
layout(parallels);
}
}


}
finally
{
//graph.getModel().setValue(parent, "NETWORKLAYOUT");
//graph.getModel().setValue(parent, "");
graph.getModel().endUpdate();
}
}

private void resetEdges() {
    Map cells = (Map)((mxGraphModel)(getGraph().getModel())).getCells();
       
    for(Object id : cells.keySet()) {
        mxCell cell = (mxCell)cells.get(id);
        if(cell instanceof mxLink) {
            if(cell.getGeometry().getPoints() != null) {
                setEdgePoints(cell, null);
            }
        }
    }
}

protected Map<String, List<Object>> findParallels(Object parent)
{
    Map<String, List<Object>> lookup = new Hashtable<String, List<Object>>();
    mxIGraphModel model = graph.getModel();

    int childCount = model.getChildCount(parent);

    for (int i = 0; i < childCount; i++) {
        Object child = model.getChildAt(parent, i);

        if (!isEdgeIgnored(child)) {
            String id = getEdgeId(child);
            if (id != null) {
                if (!lookup.containsKey(id)) {
                    lookup.put(id, new ArrayList<Object>());
                }

                lookup.get(id).add(child);
            }
        }
    }

    return lookup;
}


protected String getEdgeId(Object edge)
{
mxGraphView view = graph.getView();
mxCellState state = view.getState(edge);
Object src = (state != null) ? state.getVisibleTerminal(true) : view
.getVisibleTerminal(edge, true);
Object trg = (state != null) ? state.getVisibleTerminal(false) : view
.getVisibleTerminal(edge, false);
if (src instanceof mxICell && trg instanceof mxICell)
{
String srcId = mxCellPath.create((mxICell) src);
String trgId = mxCellPath.create((mxICell) trg);
return (srcId.compareTo(trgId) > 0) ? trgId + "-" + srcId : srcId
+ "-" + trgId;
}
return null;
}

protected void layout(List<Object> parallels)
{
Object edge = parallels.get(0);
mxIGraphModel model = graph.getModel();
mxGeometry src = model.getGeometry(model.getTerminal(edge, true));
mxGeometry trg = model.getGeometry(model.getTerminal(edge, false));

// Routes multiple loops
if (src == trg)
{
double x0 = src.getX() + src.getWidth() + this.spacing;
double y0 = src.getY() + src.getHeight() / 2;
for (int i = 0; i < parallels.size(); i++)
{
route(parallels.get(i), x0, y0);
x0 += spacing;
}
}
else if (src != null && trg != null)
{
// Routes parallel edges
/*double scx = src.getX() + src.getWidth() / 4;
double scy = src.getY() + src.getHeight() / 4;
double tcx = trg.getX() + trg.getWidth() / 4;
double tcy = trg.getY() + trg.getHeight() / 4;
double dx = tcx - scx;
double dy = tcy - scy;
double len = Math.sqrt(dx * dx + dy * dy);
double x0 = scx + dx / 4;
double y0 = scy + dy / 4;
double nx = dy * spacing / len;
double ny = dx * spacing / len;
x0 += nx * (parallels.size() - 1) / 4;
y0 -= ny * (parallels.size() - 1) / 4;*/


double p1[] = findPoint(src, trg, 2.5, spacing, parallels.size());
double p2[] = findPoint(src, trg, 1.666666666666666, spacing, parallels.size());

for (int i = 0; i < parallels.size(); i++)
{
    if(i % 2 == 1) {
        setEdgePoints(parallels.get(i), Arrays.asList(new mxPoint[] {new mxPoint(p2[0],p2[1]), new mxPoint(p1[0],p1[1])}));
    } else {
        setEdgePoints(parallels.get(i), Arrays.asList(new mxPoint[] {new mxPoint(p1[0],p1[1]), new mxPoint(p2[0],p2[1])}));
    }
    
p1[0] -= p1[2];
p1[1] += p1[3];
p2[0] -= p2[2];
p2[1] += p2[3];

}
}
}

public double[] findPoint(mxGeometry src, mxGeometry trg, double factor, int spacing, int parsize) {
    double scx = src.getX() + src.getWidth() / 2;
    double scy = src.getY() + src.getHeight() / 2;
    double tcx = trg.getX() + trg.getWidth() / 2;
    double tcy = trg.getY() + trg.getHeight() / 2;
    double dx = tcx - scx;
    double dy = tcy - scy;
    double len = Math.sqrt(dx * dx + dy * dy);
    double x0 = scx + dx / factor;
    double y0 = scy + dy / factor;
    double nx = dy * spacing / len;
    double ny = dx * spacing / len;
    x0 += nx * (parsize - 1) / 2;
    y0 -= ny * (parsize - 1) / 2;
    
    double out[] = {x0,y0,nx,ny};
    
    return out;
}

protected void route(Object edge, mxPoint point)
{
if (graph.isCellMovable(edge))
{
setEdgePoints(edge,
Arrays.asList(new mxPoint[] { point }));
}
}

protected void route(Object edge, double x, double y)
{
if (graph.isCellMovable(edge))
{
setEdgePoints(edge,
Arrays.asList(new mxPoint[] { new mxPoint(x, y) }));
}
}
}
