package dans.GUI;

import static dans.GUI.NetworkSettingsTableModel.rowNames;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import javax.swing.table.AbstractTableModel;
import dans.network.NetworkObject;
import dans.network.NetworkObjectListener;
import dans.network.Processor;
import dans.util.Log;

/**
 *
 * @author Dan
 * @todo Refactor this and LinkSettingsTableModel into one class.
 * @todo Add tool tips.
 */
public class ProcessorSettingsTableModel extends AbstractTableModel implements NetworkObjectListener  {
    Object[][] rowData;
    public static final String[] columnNames = {"Setting Name", "Value"};
    public static final String rowNames[] = {
        "ID",
        "Delay",
        "",
        "Speed Change Method",
        "Change Seed",
        "Change Min",
        "Change Max",
        "Normal Change Mean",
        "Normal Change STDV",
        "Uniform Change Min",
        "Uniform Change Max",
        "Constant Change Amount",
        "",
        "Clean Failure Rate",
        "Byzantine Failure Rate"
    };
    private GUIHandler hgui;
    private Processor p;

    public static final Set<String> typeDouble = new HashSet<>(Arrays.asList(
        new String[] {"Clean Failure Rate", "Byzantine Failure Rate"}
    ));
    
    public static final Set<String> typeNeg = new HashSet<>(Arrays.asList(
        new String[] {"Normal Change Mean", "Uniform Change Min", "Uniform Change Max", "Constant Change Amount"}
    ));
    
    public static final Set<String> typeMethod = new HashSet<>(Arrays.asList(
        new String[] {"Speed Change Method"}
    ));
    
    public ProcessorSettingsTableModel(GUIHandler hgui, Processor proc) {
        rowData = proc.dumpSettings(ProcessorSettingsTableModel.rowNames);
        this.hgui = hgui;
        p = proc;
        p.addListener(this, true);
    }
    
    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    @Override
    public String getColumnName(int column) {
        return columnNames[column];
    }

    @Override
    public int getRowCount() {
        return rowData.length;
    }

    @Override
    public Object getValueAt(int row, int column) {
        return rowData[row][column];
    }

    @Override
    public Class getColumnClass(int column) {
        if(column == 0) {
            return String.class;
        } else {
            return Integer.class;
        }
    }

    @Override
    public void setValueAt(Object value, int row, int column) {  
        if(!rowData[row][column].equals(value)) {
            rowData[row][column] = value;
            fireTableCellUpdated(row, column);
            
            if(value instanceof String) {
                value = ((String)value).replaceAll(",", "");
            }
            
            hgui.processorSettingsUpdate(rowNames[row], value, p);
        }
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        return column == 1 && !ProcessorSettingsTableModel.rowNames[row].equals("");
    } 

    @Override
    public void networkObjectChange(NetworkObject object, Object oldvalue, int eventID, Object caller) {
        if(caller != hgui) {
            if(eventID == NetworkObject.EVENT_SETTINGS_UPDATE) {
                String name = (String)((Object[])oldvalue)[0];
                int row = Arrays.asList(rowNames).indexOf(name);
                if(row >= 0) {
                    rowData[row][1] = object.getSetting(name);
                    fireTableCellUpdated(row, 1);
                }
            } else if(eventID == NetworkObject.EVENT_ID_UPDATE) {
                int row = Arrays.asList(rowNames).indexOf("ID");
                if(row >= 0) {
                    rowData[row][1] = object.getSetting(object.getID());
                    fireTableCellUpdated(row, 1);
                }
            }
        }
    }
    
    public void removeListener() {
        p.removeListener(this);
    }
}
