package dans.GUI;

import com.mxgraph.model.mxCell;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.Arrays;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import dans.network.Link;
import dans.network.NetworkObject;
import dans.network.NetworkState;
import dans.network.Processor;

/**
 *
 * @author Dan
 */
public class PropertiesTab extends JTabbedPane {
    private GUIHandler hgui;

    private JComponent object;
    private JPanel objectPanel;
    private JLabel objectHeader;
    private JTable objectTable;
    private TableModel objectTableModel;
    
    private JComponent network;
    private JTable networkTable;
    private TableModel networkTableModel;
    
    private JComponent stats;
    private JLabel statsHeader;
    private JTable statsNetworkTable;
    private TableModel statsNetworkTableModel;
    private JTable objectStatsTable;
    private ObjectStatsTableModel objectStatsTableModel;
    
    private JComponent algorithms;
    private JLabel algorithmsHeader;
    private JTable algorithmsTable;
    private TableModel algorithmsTableModel;
    private JPanel algorithmsPanel;
    
    private boolean inLockDown = false;
    
    private static final String procStatNames[] = {
        "Status",
        "",
        "Messages",
        "Messages in Bytes",
        "Cycles"
    };
    
    private static final String linkStatNames[] = {
        "Status",
        "",
        "Messages",
        "Messages in Bytes",
        "Messages per Tick",
        "Bytes per Tick",
        "",
        "Packets Lost",
        "Packet Errors"
    };
    
    public PropertiesTab(GUIHandler hgui, NetworkState state) {
        super();
        
        this.hgui = hgui;
        
        Object settingsData[][] = state.dumpSettings(NetworkSettingsTableModel.rowNames);
        
        object = new JPanel();
        object.setBorder(BorderFactory.createLineBorder(Color.GRAY));
        object.setLayout(new BorderLayout());
        objectHeader = new JLabel("Select a Processor or Link");
        objectHeader.setFont(new Font(objectHeader.getFont().getName(), Font.PLAIN, 17));
        objectHeader.setBorder(BorderFactory.createEmptyBorder(5,2,5,0));
        objectPanel = new JPanel();
        objectPanel.setBorder(null);
        object.add(objectHeader, BorderLayout.PAGE_START);
        object.add(new JScrollPane(objectPanel), BorderLayout.CENTER);
        
        networkTableModel = new NetworkSettingsTableModel(hgui, settingsData);
        networkTable = new JTable(networkTableModel);
        networkTable.getTableHeader().setReorderingAllowed(false);
        TableColumn col1 = networkTable.getColumnModel().getColumn(0);
        col1.setMinWidth(85);
        col1.setPreferredWidth(175);
        TableColumn col2 = networkTable.getColumnModel().getColumn(1);
        col2.setCellEditor(new NetworkSettingsCellEditor());
        col2.setPreferredWidth(60);
        col2.setMinWidth(35);
        network = new JScrollPane(networkTable);
        
        

        stats = new JPanel();
        stats.setBorder(BorderFactory.createLineBorder(Color.GRAY));
        stats.setLayout(new BorderLayout());
        Object statsData[][] = state.dumpStats(NetworkStatsTableModel.rowNames);
        statsNetworkTableModel = new NetworkStatsTableModel(statsData);
        statsNetworkTable = new JTable(statsNetworkTableModel);
        statsNetworkTable.getTableHeader().setReorderingAllowed(false);
        col1 = statsNetworkTable.getColumnModel().getColumn(0);
        col1.setMinWidth(85);
        col1.setPreferredWidth(150);
        col2 = statsNetworkTable.getColumnModel().getColumn(1);
        //col2.setCellEditor(new NetworkSettingsCellEditor());
        col2.setPreferredWidth(60);
        col2.setMinWidth(35);
        statsHeader = new JLabel("Global Network Statistics");
        statsHeader.setFont(new Font(statsHeader.getFont().getName(), Font.PLAIN, 17));
        statsHeader.setBorder(BorderFactory.createEmptyBorder(5,2,5,0));
        stats.add(statsHeader, BorderLayout.PAGE_START);
        stats.add(new JScrollPane(statsNetworkTable), BorderLayout.CENTER);
        
        
        algorithms = new JPanel();
        algorithms.setBorder(BorderFactory.createLineBorder(Color.GRAY));
        algorithms.setLayout(new BorderLayout());
        algorithmsHeader = new JLabel("Select a Processor");
        algorithmsHeader.setFont(new Font(algorithmsHeader.getFont().getName(), Font.PLAIN, 17));
        algorithmsHeader.setBorder(BorderFactory.createEmptyBorder(5,2,5,0));
        algorithmsPanel = new JPanel();
        algorithmsPanel.setBorder(null);
        algorithms.add(algorithmsHeader, BorderLayout.PAGE_START);
        algorithms.add(new JScrollPane(algorithmsPanel), BorderLayout.CENTER);
        
        
        addTab("Statistics", null, stats, "View golbal network statistic");
        addTab("Properties", null, object, "Edit/view selected processor or link properties");
        addTab("Algorithms", null, algorithms, "Edit/view algorithms for a given processor");
        addTab("Defaults", null, network, "Edit/view golbal settings and defaults");
        
        this.setFont(new Font(this.getFont().getName(), Font.BOLD, 11));
        
        this.setMinimumSize(new Dimension(261,75));
        
        hgui.register(this);
    }
    
    public void setSettingsValue(String name, Object value) {
        int index = Arrays.asList(NetworkSettingsTableModel.rowNames).indexOf(name);
        if(index >= 0) {
            networkTableModel.setValueAt(value, index, 1);
        }
    }
    
    public void displayStatsFor(mxCell cell) {
        stats.removeAll();
        stats.add(statsHeader, BorderLayout.PAGE_START);
        
        if(objectStatsTableModel != null) {
            objectStatsTableModel.removeListener();
        }
        
        if(cell == null) {
            statsHeader.setText("Global Network Statistics");
            stats.add(new JScrollPane(statsNetworkTable), BorderLayout.CENTER);
            
        } else {
            if(cell instanceof mxProcessor) {
                Processor p = ((mxProcessor)cell).getProcessor();
                objectStatsTableModel = new ObjectStatsTableModel(p, procStatNames);
                objectStatsTable = new JTable(objectStatsTableModel);
                statsHeader.setText("Statistics for Processor " + p.getID());
            } else if(cell instanceof mxLink) {
                Link l = ((mxLink)cell).getLink();
                objectStatsTableModel = new ObjectStatsTableModel(l, linkStatNames);
                objectStatsTable = new JTable(objectStatsTableModel);
                statsHeader.setText("Statistics for Link " + l.getID());
            }
            
            objectStatsTable.getTableHeader().setReorderingAllowed(false);   
            TableColumn col1 = objectStatsTable.getColumnModel().getColumn(0);
            col1.setMinWidth(85);
            col1.setPreferredWidth(145);
            TableColumn col2 = objectStatsTable.getColumnModel().getColumn(1);
            col2.setPreferredWidth(60);
            col2.setMinWidth(35);
            objectStatsTable.setEnabled(!inLockDown);
            stats.add(new JScrollPane(objectStatsTable), BorderLayout.CENTER);
        }
        
        stats.validate();
        stats.repaint();
    }
    
    public void displayAlgorithmsFor(mxProcessor mxp) {
        algorithms.removeAll();
        algorithms.add(algorithmsHeader, BorderLayout.PAGE_START);
        
        if(algorithmsTable != null) {
            TableColumn col = algorithmsTable.getColumnModel().getColumn(0);
            col.getCellEditor().stopCellEditing();
            col = algorithmsTable.getColumnModel().getColumn(1);
            col.getCellEditor().stopCellEditing();
            
            if(algorithmsTable.getModel() instanceof AlgorithmsTableModel) {
                ((AlgorithmsTableModel)algorithmsTable.getModel()).removeListener();
            }
                    
            algorithmsTable = null;
            algorithmsTableModel = null;
        }
        
        if(mxp != null) {
            algorithmsHeader.setText("Algorithms for Processor " + mxp.getValue());
            algorithmsTableModel = new AlgorithmsTableModel(hgui, mxp);
            algorithmsTable = new JTable(algorithmsTableModel);
            algorithmsTable.getTableHeader().setReorderingAllowed(false);
            AlgorithmsCellEditor editor = new AlgorithmsCellEditor(hgui, (AlgorithmsTableModel) algorithmsTableModel);
            TableColumn col1 = algorithmsTable.getColumnModel().getColumn(0);
            col1.setCellEditor(editor);
            col1.setMinWidth(25);
            col1.setPreferredWidth(40);
            TableColumn col2 = algorithmsTable.getColumnModel().getColumn(1);
            col2.setCellEditor(editor);
            col2.setPreferredWidth(150);
            col2.setMinWidth(85);
            TableColumn col3 = algorithmsTable.getColumnModel().getColumn(2);
            col3.setCellRenderer(new AlgorithmsButtonRenderer((AlgorithmsTableModel) algorithmsTableModel));
            col3.setCellEditor(editor);
            col3.setPreferredWidth(16);
            col3.setMinWidth(16);
            algorithmsTable.setEnabled(!inLockDown);
            algorithms.add(new JScrollPane(algorithmsTable), BorderLayout.CENTER);
        } else {
            algorithms.add(algorithmsPanel, BorderLayout.CENTER);
            algorithmsHeader.setText("Select a Processor");
        }
        
        algorithms.validate();
        algorithms.repaint();
    }
    
    public void displaySettingsFor(mxCell cell) {
        object.removeAll();
        object.add(objectHeader, BorderLayout.PAGE_START);

        if(objectTable != null) {
            TableColumn col2 = objectTable.getColumnModel().getColumn(1);
            col2.getCellEditor().stopCellEditing();
            
            if(objectTable.getModel() instanceof ProcessorSettingsTableModel) {
                ((ProcessorSettingsTableModel)objectTable.getModel()).removeListener();
            } else if(objectTable.getModel() instanceof LinkSettingsTableModel) {
                ((LinkSettingsTableModel)objectTable.getModel()).removeListener();
            }
                    
            objectTable = null;
            objectTableModel = null;
        }
        
        
        if(cell instanceof mxProcessor) {
            objectHeader.setText("Settings for Processor " + cell.getValue());
            objectTableModel = new ProcessorSettingsTableModel(hgui, ((mxProcessor)cell).getProcessor());
            objectTable = new JTable(objectTableModel);
            objectTable.getTableHeader().setReorderingAllowed(false);
            TableColumn col1 = objectTable.getColumnModel().getColumn(0);
            col1.setMinWidth(85);
            col1.setPreferredWidth(175);
            TableColumn col2 = objectTable.getColumnModel().getColumn(1);
            col2.setCellEditor(new ProcessorSettingsCellEditor());
            col2.setPreferredWidth(60);
            col2.setMinWidth(35);
            objectTable.setEnabled(!inLockDown);
            object.add(new JScrollPane(objectTable), BorderLayout.CENTER);
        } else if (cell instanceof mxLink) {
            objectHeader.setText("Settings for Link " + ((mxLink)cell).getLink().getID());
            objectTableModel = new LinkSettingsTableModel(hgui, ((mxLink)cell).getLink());
            objectTable = new JTable(objectTableModel);
            objectTable.getTableHeader().setReorderingAllowed(false);
            TableColumn col1 = objectTable.getColumnModel().getColumn(0);
            col1.setMinWidth(85);
            col1.setPreferredWidth(175);
            TableColumn col2 = objectTable.getColumnModel().getColumn(1);
            col2.setCellEditor(new LinkSettingsCellEditor());
            col2.setPreferredWidth(60);
            col2.setMinWidth(35);
            objectTable.setEnabled(!inLockDown);
            object.add(new JScrollPane(objectTable), BorderLayout.CENTER);
        } else {
            object.add(objectPanel, BorderLayout.CENTER);
            objectHeader.setText("Select a Processor or Link");
        }
        
        object.validate();
        object.repaint();
    }

    void setStatsValue(String name, Object value) {
        int index = Arrays.asList(NetworkStatsTableModel.rowNames).indexOf(name);
        if(index >= 0) {
            statsNetworkTableModel.setValueAt(value, index, 1);
        }
    }

    void lockUpForPlay() {
        inLockDown = true;
        
        networkTable.setEnabled(false);
        statsNetworkTable.setEnabled(false);
        
        if(algorithmsTable != null) {
            algorithmsTable.setEnabled(false);
        }
        
        if(objectStatsTable != null) {
            objectStatsTable.setEnabled(false);
        }
        
        if(objectTable != null) {
            objectTable.setEnabled(false);
        }
    }

    void unlockUpForReset() {
        inLockDown = false;
        
        networkTable.setEnabled(true);
        statsNetworkTable.setEnabled(true);
        
        if(algorithmsTable != null) {
            algorithmsTable.setEnabled(true);
        }
        
        if(objectStatsTable != null) {
            objectStatsTable.setEnabled(true);
        }
        
        if(objectTable != null) {
            objectTable.setEnabled(true);
        }
    }

    void reset(NetworkState state) {
        Object statsData[][] = state.dumpStats(NetworkStatsTableModel.rowNames);
        statsNetworkTableModel = new NetworkStatsTableModel(statsData);
        statsNetworkTable.setModel(statsNetworkTableModel);
        statsNetworkTable.getTableHeader().setReorderingAllowed(false);
        TableColumn col1 = statsNetworkTable.getColumnModel().getColumn(0);
        col1.setMinWidth(85);
        col1.setPreferredWidth(150);
        TableColumn col2 = statsNetworkTable.getColumnModel().getColumn(1);
        col2.setPreferredWidth(60);
        col2.setMinWidth(35);
        
        Object settingsData[][] = state.dumpSettings(NetworkSettingsTableModel.rowNames);
        networkTableModel = new NetworkSettingsTableModel(hgui, settingsData);
        networkTable.setModel(networkTableModel);
        networkTable.getTableHeader().setReorderingAllowed(false);
        col1 = networkTable.getColumnModel().getColumn(0);
        col1.setMinWidth(85);
        col1.setPreferredWidth(175);
        col2 = networkTable.getColumnModel().getColumn(1);
        col2.setCellEditor(new NetworkSettingsCellEditor());
        col2.setPreferredWidth(60);
        col2.setMinWidth(35);
        
        
        displaySettingsFor(null);
        displayAlgorithmsFor(null);
        displayStatsFor(null);
    }
}
