package dans.GUI;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JToolBar;

/**
 *
 * @author Dan
 */
public class ToolBar extends JToolBar{
    public static final int EDIT_MODE_DISABLED = 0;
    public static final int EDIT_MODE_SELECT = 1;
    public static final int EDIT_MODE_PROC = 2;
    public static final int EDIT_MODE_LINK = 3;
    public static final int EDIT_MODE_DELETE = 4;
    public static final int EDIT_MODE_PAN = 5;
    
    private int editMode = EDIT_MODE_SELECT;
    
    private GUIHandler hgui;
    
    private JButton newButton;
    private JButton openButton;
    private JButton saveButton;
    
    private JButton undoButton;
    private JButton redoButton;
    
    private JButton clickButton;
    private JButton panButton;
    private JButton procButton;
    private JButton linkButton;
    private JButton delButton;
    
    private JButton playButton;
    private JButton pauseButton;
    private JButton restartButton;
    
    private JButton showLog;
    private JButton showProps;
    private JButton showGrid;
    
    private JButton zoomIn;
    private JButton zoomOut;
    
    private JButton load;
    private JComboBox defaultAlgo;
    private JLabel defaultLabel;
            
    private Color defaultColor;
    
    public ToolBar(GUIHandler hgui) {
        super();
        
        this.hgui = hgui;
        
        
        newButton = new JButton();
        newButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/New24.gif")));
        newButton.setToolTipText("New network");
        newButton.setSize(24, 24);
        newButton.addActionListener((ActionEvent e) -> hgui.newNetwork());
        this.add(newButton);
        
        defaultColor = newButton.getBackground();
        
        openButton = new JButton();
        openButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Open24.gif")));
        openButton.setToolTipText("Open network");
        openButton.setSize(24, 24);
        openButton.addActionListener((ActionEvent e) -> hgui.load());
        this.add(openButton);
        
        saveButton = new JButton();
        saveButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Save24.gif")));
        saveButton.setToolTipText("Save network");
        saveButton.setSize(24, 24);
        saveButton.addActionListener((ActionEvent e) -> hgui.save());
        this.add(saveButton);
        
        this.addSeparator();
        
        undoButton = new JButton();
        undoButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Undo24.gif")));
        undoButton.setToolTipText("Undo last edit");
        undoButton.setSize(24, 24);
        undoButton.addActionListener((ActionEvent e) -> {
            hgui.undo();
        });
        this.add(undoButton);
        
        redoButton = new JButton();
        redoButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Redo24.gif")));
        redoButton.setToolTipText("Redo last undone edit");
        redoButton.setSize(24, 24);
        redoButton.addActionListener((ActionEvent e) -> {
            hgui.redo();
        });
        this.add(redoButton);
        
        this.addSeparator();
        
        clickButton = new JButton();
        clickButton.setIcon(new ImageIcon(this.getClass().getResource("/dans/imgs/arrow.png")));
        clickButton.setToolTipText("Move/edit network objects");
        clickButton.setSize(24, 24);
        clickButton.setBackground(Color.WHITE);
        clickButton.addActionListener((ActionEvent e) -> {
            hgui.editMode(ToolBar.EDIT_MODE_SELECT);
        });
        this.add(clickButton);
        
        panButton = new JButton();
        panButton.setIcon(new ImageIcon(this.getClass().getResource("/dans/imgs/panning.png")));
        panButton.setToolTipText("Pan graph");
        panButton.setSize(24, 24);
        panButton.addActionListener((ActionEvent e) -> {
            hgui.editMode(ToolBar.EDIT_MODE_PAN);
        });
        this.add(panButton);
        
        procButton = new JButton();
        procButton.setIcon(new ImageIcon(this.getClass().getResource("/dans/imgs/node.png")));
        procButton.setToolTipText("Add processor");
        procButton.setSize(24,24);
        procButton.addActionListener((ActionEvent e) -> {
            hgui.editMode(ToolBar.EDIT_MODE_PROC);
        });
        this.add(procButton);
        
        linkButton = new JButton();
        linkButton.setIcon(new ImageIcon(this.getClass().getResource("/dans/imgs/link.png")));
        linkButton.setToolTipText("Add link");
        linkButton.setSize(24,24);
        linkButton.addActionListener((ActionEvent e) -> {
            hgui.editMode(ToolBar.EDIT_MODE_LINK);
        });
        this.add(linkButton);
        
        delButton = new JButton();
        delButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Delete24.gif")));
        delButton.setToolTipText("Delete network object");
        delButton.setSize(24,24);
        delButton.addActionListener((ActionEvent e) -> {
            //hgui.delete(e); //If uncomeneted causes the currently selected processor to be deleted.
            hgui.editMode(ToolBar.EDIT_MODE_DELETE);
        });
        this.add(delButton);
        
        this.addSeparator();
        
        playButton = new JButton();
        playButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/media/Play24.gif")));
        playButton.setToolTipText("Play simulation");
        playButton.setSize(24,24);
        playButton.addActionListener((ActionEvent e) -> hgui.play());
        this.add(playButton);
        
        pauseButton = new JButton();
        pauseButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/media/Pause24.gif")));
        pauseButton.setToolTipText("Pause simulation");
        pauseButton.setEnabled(false);
        pauseButton.setSize(24,24);
        pauseButton.addActionListener((ActionEvent e) -> hgui.pause());
        this.add(pauseButton);
        
        restartButton = new JButton();
        restartButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/media/StepBack24.gif")));
        restartButton.setToolTipText("Restart simulation");
        restartButton.setEnabled(false);
        restartButton.setSize(24,24);
        restartButton.addActionListener((ActionEvent e) -> hgui.reset());
        this.add(restartButton); 
        
        this.addSeparator();
        
        showLog = new JButton();
        showLog.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Information24.gif")));
        showLog.setToolTipText("Show log console");
        showLog.setSize(24, 24);
        showLog.addActionListener((ActionEvent e) -> {
            hgui.showLogWindow();
        });
        this.add(showLog);
        
        showProps = new JButton();
        showProps.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Preferences24.gif")));
        showProps.setToolTipText("Show properties tab");
        showProps.setBackground(Color.WHITE);
        showProps.setSize(24, 24);
        showProps.addActionListener((ActionEvent e) -> {
            hgui.showPropsTab();
        });
        this.add(showProps);
        
        showGrid = new JButton();
        showGrid.setIcon(new ImageIcon(this.getClass().getResource("/dans/imgs/grid.gif")));
        showGrid.setToolTipText("Show grid overlay");
        showGrid.setSize(24, 24);
        showGrid.addActionListener((ActionEvent e) -> {
            hgui.showGrid();
        });
        this.add(showGrid);
        
        this.addSeparator();
        
        zoomIn = new JButton();
        zoomIn.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/ZoomIn24.gif")));
        zoomIn.setToolTipText("Zoom in");
        zoomIn.setSize(24, 24);
        zoomIn.addActionListener((ActionEvent e) -> {
            hgui.zoomIn();
        });
        this.add(zoomIn);
        
        zoomOut = new JButton();
        zoomOut.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/ZoomOut24.gif")));
        zoomOut.setToolTipText("Zoom in");
        zoomOut.setSize(24, 24);
        zoomOut.addActionListener((ActionEvent e) -> {
            hgui.zoomOut();
        });
        this.add(zoomOut);
        
        this.addSeparator();
        
        load = new JButton("Load Algorithm");
        load.addActionListener((ActionEvent e) -> {
            hgui.loadAlgorithm();
        });
        load.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Import24.gif")));
        this.add(load);
        
        this.addSeparator();
        
        defaultLabel = new JLabel("Default: ");
        this.add(defaultLabel);
        
        defaultAlgo = new JComboBox(new String[]{"<None>"});
        defaultAlgo.setPreferredSize(new Dimension(150,25));
        defaultAlgo.setMaximumSize(defaultAlgo.getPreferredSize());
        defaultAlgo.addActionListener((ActionEvent e) -> {
            if(defaultAlgo.getSelectedItem() != null) {
                hgui.setDefaultAlgorithm(defaultAlgo.getSelectedItem().toString());
            }
        });
        this.add(defaultAlgo);
        
        this.setFloatable(false);
        hgui.register(this);
    }

    public void setLogToggle(boolean b) {
        if(b) {
            showLog.setBackground(Color.WHITE);
        } else {
            showLog.setBackground(defaultColor);
        }
    }

    public void setPropsToggle(boolean b) {
        if(b) {
            showProps.setBackground(Color.WHITE);
        } else {
            showProps.setBackground(defaultColor);
        }
    }

    public void setGridToggle(boolean b) {
        if(b) {
            showGrid.setBackground(Color.WHITE);
        } else {
            showGrid.setBackground(defaultColor);
        }
    }
    
    private void resetEditModeBackgrounds() {
        clickButton.setBackground(defaultColor);
        procButton.setBackground(defaultColor);
        linkButton.setBackground(defaultColor);
        delButton.setBackground(defaultColor);
        panButton.setBackground(defaultColor);
    }
    
    private void disableAllEditMode() {
        clickButton.setEnabled(false);
        procButton.setEnabled(false);
        linkButton.setEnabled(false);
        delButton.setEnabled(false);
        panButton.setEnabled(false);
    }
    
    private void enableAllEditMode() {
        clickButton.setEnabled(true);
        procButton.setEnabled(true);
        linkButton.setEnabled(true);
        delButton.setEnabled(true);
        panButton.setEnabled(true);
    }
    
    public void setEditMode(int mode) {
        resetEditModeBackgrounds();
        //enableAllEditMode();
        editMode = mode;
        switch(mode) {
            case ToolBar.EDIT_MODE_DISABLED:
                disableAllEditMode();
                break;
            case ToolBar.EDIT_MODE_SELECT:
                clickButton.setBackground(Color.WHITE);
                break;
            case ToolBar.EDIT_MODE_PROC:
                procButton.setBackground(Color.WHITE);
                break;
            case ToolBar.EDIT_MODE_LINK:
                linkButton.setBackground(Color.WHITE);
                break;
            case ToolBar.EDIT_MODE_DELETE:
                delButton.setBackground(Color.WHITE);
                break;
            case ToolBar.EDIT_MODE_PAN:
                panButton.setBackground(Color.WHITE);
                break;
            default:
                setEditMode(ToolBar.EDIT_MODE_SELECT);
        }
    }
    
    public int getEditMode() {
        return editMode;
    }

    public void addAlgorithm(Class algorithm, boolean select) {
        defaultAlgo.addItem(algorithm.getSimpleName());
        
        if(select) {
            defaultAlgo.setSelectedItem(algorithm.getSimpleName());
        }
    }
    
    public void removeAlgorithm(Class algorithm) {
        if(defaultAlgo.getSelectedItem().equals(algorithm.getSimpleName())) {
            defaultAlgo.setSelectedIndex(0);
        }
        
        defaultAlgo.removeItem(algorithm.getSimpleName());
    }

    void lockUpForPlay() {
        newButton.setEnabled(false);
        openButton.setEnabled(false);
        saveButton.setEnabled(false);
    
        undoButton.setEnabled(false);
        redoButton.setEnabled(false);

        procButton.setEnabled(false);
        linkButton.setEnabled(false);
        delButton.setEnabled(false);
    
        playButton.setEnabled(false);
        playButton.setBackground(Color.WHITE);
        pauseButton.setEnabled(true);
        pauseButton.setBackground(defaultColor);
        restartButton.setEnabled(false);
    
        load.setEnabled(false);
        defaultAlgo.setEnabled(false);
        defaultLabel.setEnabled(false);
    }

    void lockUpForPause() {
        playButton.setEnabled(true);
        playButton.setBackground(defaultColor);
        pauseButton.setEnabled(false);
        pauseButton.setBackground(Color.WHITE);
        restartButton.setEnabled(true);
    }

    void unlockUpForReset() {
        newButton.setEnabled(true);
        openButton.setEnabled(true);
        saveButton.setEnabled(true);
    
        undoButton.setEnabled(true);
        redoButton.setEnabled(true);

        procButton.setEnabled(true);
        linkButton.setEnabled(true);
        delButton.setEnabled(true);
    
        playButton.setEnabled(true);
        playButton.setBackground(defaultColor);
        pauseButton.setEnabled(false);
        pauseButton.setBackground(defaultColor);
        restartButton.setEnabled(false);
    
        load.setEnabled(true);
        defaultAlgo.setEnabled(true);
        defaultLabel.setEnabled(true);
    }

    void lockUpForStop() {
        playButton.setEnabled(false);
        playButton.setBackground(defaultColor);
        pauseButton.setEnabled(false);
        pauseButton.setBackground(defaultColor);
        restartButton.setEnabled(true);
    }

    void clearAlgorithms() {
        defaultAlgo.removeAllItems();
        defaultAlgo.addItem("<None>");
        defaultAlgo.setSelectedIndex(0);
    }
}
