package dans.GUI;

import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxGraphModel;
import com.mxgraph.model.mxICell;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;
import dans.network.Link;
import dans.network.NetworkObject;
import dans.network.NetworkObjectListener;
import dans.network.Processor;
import dans.util.Log;
import java.util.Map;

/**
 *
 * @author Dan
 */
public class mxLink extends mxCell  implements NetworkObjectListener {
    public static final String STYLE_ACTIVE = "edgeon";
    public static final String STYLE_INACTIVE = "edgeoff";
    public static final String STYLE_CRASH = "edgefail";

    
    transient private Link link;
    private String lastUsedID = null;
    
    private static ConcurrentHashMap<String, Link> linkBackup = new ConcurrentHashMap<>();
    private static int lastID = 0;
    
    private mxCell sourceLabel = null;
    private mxCell targetLabel = null;
    
    public boolean loading = false;
    
    public static synchronized int getLastID() {
        return lastID++;
    }
    
    public static synchronized void setLastID(int id) {
        lastID = id;
    }
    
    public boolean isComplete() {
        return link != null && link.isComplete() && this.getSource() != null && this.getTarget() != null;
    }
    
    public mxLink(Link l) {
        this(l, null, STYLE_INACTIVE);
    }
    
    public mxLink(Link l, mxGeometry geometry) {
        this(l, geometry, STYLE_INACTIVE);
    }
    
    public mxLink(Link l, mxGeometry geometry, String style) {
        super("", geometry, style);
        link = l;
        this.vertex = false;
        this.edge = true;
        l.addListener(this, true);
        setUpLabels();
    }
    
    public void setSourceLabel(String text) {
        if(sourceLabel != null) {
            synchronized(sourceLabel) {
                sourceLabel.setValue(text);
            }
        }
    }
    
    public void setTargetLabel(String text) {
        if(targetLabel != null) {
            synchronized(targetLabel) {
                targetLabel.setValue(text);
            }
        }
    }
    
    private void setUpLabels() {
        mxGeometry geo = new mxGeometry(-0.7, 0, 0, 0);
        geo.setRelative(true);
        sourceLabel = new mxCell("", geo, "sourcelabel");
        sourceLabel.setConnectable(false);
        sourceLabel.setVertex(true);
        this.insert(sourceLabel);
        
        geo = new mxGeometry(0.7, 0, 0, 0);
        geo.setRelative(true);
        targetLabel = new mxCell("", geo, "targetlabel");
        targetLabel.setConnectable(false);
        targetLabel.setVertex(true);
        this.insert(targetLabel);
    }
    
    public Link getLink() {
        return link;
    }
    
    public void setLink(Link l) {
        this.link = l;
    }
    
    @Override
    public boolean isVertex() {
        return false;
    }
    
    @Override
    public void setVertex(boolean b) {
        super.setVertex(false);
    }
    
    @Override
    public boolean isEdge() {
        return true;
    }
    
    @Override
    public void setEdge(boolean b) {
        super.setVertex(true);
    }
    
    /*@Override
    public void setValue(Object lid) {
        if(link != null) { //Will be null when getValue is called in mxCell constructor
            link.setID(lid.toString(), this);
        }
        
        super.setValue(lid);
    } */ 
    
    /*@Override
    public Object getValue(){
        return link.getID();
    }*/

    @Override
    public void networkObjectChange(NetworkObject object, Object oldvalue, int eventID, Object caller) {
        Link l = (Link)object;

        /*if(eventID == NetworkObject.EVENT_ID_UPDATE) {
            this.value = l.getID();
        }*/
    }
    
    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        link = new Link(mxLink.getLastID() + "", linkBackup.get(lastUsedID));
        lastUsedID = null;
    }
    
    private void writeObject(ObjectOutputStream stream) throws IOException {
        lastUsedID = link.getID();
        linkBackup.put(lastUsedID, link);
        stream.defaultWriteObject();
    }
    
    public static void clearBackup() {
        linkBackup.clear();
    }
    
    static void clearBackup(String id) {
        linkBackup.remove(id);
    }
    
    @Override
    public void setSource(mxICell source){
        if(loading) {
            super.setSource(source);
        } else {
            if(source != getSource()) {
                super.setSource(source);
                if(source instanceof mxProcessor) {
                    link.setSource(((mxProcessor)source).getProcessor());
                }
            }
        }
    }
    
    @Override
    public void setTarget(mxICell target){
        if(loading) {
            super.setTarget(target);
        } else {   
            if(target != getTarget()) {
                super.setTarget(target);
                if(target instanceof mxProcessor) {
                    link.setTarget(((mxProcessor)target).getProcessor());
                }
            }
        }
    }
    
    public void removeListener() {
        link.removeListener(this);
    }
    
    public static mxLink findLink(Link l, mxGraphModel model) {
        Map cells = model.getCells();
        for(Object cell : cells.values()) {
            if(cell instanceof mxLink) {
                if(((mxLink)cell).getLink() == l) {
                    return (mxLink)cell;
                }
            }
        }
        
        return null;
    }
}