package dans.GUI;

import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxGraphModel;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamException;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import dans.network.NetworkObject;
import dans.network.NetworkObjectListener;
import dans.network.Processor;
import java.util.Map;

/**
 *
 * @author Dan
 */
public class mxProcessor extends mxCell implements NetworkObjectListener {
    public static final int DEFAULT_SIZE = 75;
    
    public static final String STYLE_ACTIVE = "running";
    public static final String STYLE_ACTIVE_ROOT = "runningroot";
    public static final String STYLE_INACTIVE = "stopped";
    public static final String STYLE_INACTIVE_ROOT = "stoppedroot";
    public static final String STYLE_CRASH = "failed";
    public static final String STYLE_CRASH_ROOT = "failedroot";
    
    private static int lastID = 0;
    
    transient private Processor processor;
    
    private static ConcurrentHashMap<String, Processor> processorBackup = new ConcurrentHashMap<>();
    
    private mxCell topLabel = null;
    private mxCell bottomLabel = null;
    
    public boolean loading = false;
    
    public static synchronized int getLastID() {
        return lastID++;
    }
    
    public static synchronized void setLastID(int id) {
        lastID = id;
    }
    
    public mxProcessor(Processor p) {
        this(p, null);
    }
    
    public mxProcessor(Processor p, mxGeometry geometry) {
        super(p.getID(), geometry, STYLE_ACTIVE);
        processor = p;
        this.vertex = true;
        this.edge = false;
        p.addListener(this, true);
        setUpLabels();
    }

    public void setTopLabel(String text) {
        if(topLabel != null) {
            synchronized(topLabel) {
                topLabel.setValue(text);
            }
        }
    }
    
    public void setBottomLabel(String text) {
        if(bottomLabel != null) {
            synchronized(bottomLabel) {
                bottomLabel.setValue(text);
            }
        }
    }
    
    private void setUpLabels() {
        mxGeometry geo = new mxGeometry(0, -0.1, 0, 0);
        geo.setRelative(true);
        topLabel = new mxCell("", geo, "toplabel");
        topLabel.setConnectable(false);
        topLabel.setVertex(true);
        this.insert(topLabel);
        
        geo = new mxGeometry(1, 1.1, 0, 0);
        geo.setRelative(true);
        bottomLabel = new mxCell("", geo, "bottomlabel");
        bottomLabel.setConnectable(false);
        bottomLabel.setVertex(true);
        this.insert(bottomLabel);
    }
    
    public Processor getProcessor() {
        /*if(processor == null) {
            copyCheck();
        }*/
        
        return processor;
    }
    
    @Override
    public boolean isVertex() {
        return true;
    }
    
    @Override
    public void setVertex(boolean b) {
        super.setVertex(true);
    }
     
    @Override
    public boolean isEdge() {
        return false;
    }
    
    @Override
    public void setEdge(boolean b) {
        super.setVertex(false);
    }
    
    @Override
    public void setValue(Object pid) {
        if(processor != null) {
            processor.setID(pid.toString(), this);
        }
        
        super.setValue(pid);
    }  

    @Override
    public Object getValue() {
        /*if(processor == null) {
            copyCheck();
        }*/
                
        return processor.getID();
    }
    
    @Override
    public void networkObjectChange(NetworkObject object, Object oldvalue, int eventID, Object caller) {
        Processor p = (Processor)object;
        
        if(eventID == NetworkObject.EVENT_ID_UPDATE) {
            this.value = p.getID();
        } /*else if(eventID == NetworkObject.EVENT_STATUS_UPDATE) {
            if(p.getStatus().equals(NetworkObject.STATUS_ERROR)) {
                setStyle("stopped");
            }
        }*/
    }
    
    /*private synchronized void copyCheck() 
        System.out.println(id + "_" + value);
        processor = processorBackup.remove(id + "_" + value);
        System.out.println(processor);
        
        if(processor == null) {
            processor = new Processor("Copy");
            processor.setID("Copy [" + id + "]", this);
            value = "Copy [" + id + "]";
            System.out.println(value);
        }
    }*/
    
    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        processor = processorBackup.get(value);
    }
    
    private void writeObject(ObjectOutputStream stream) throws IOException {
        processorBackup.put(getValue().toString(), processor);
        stream.defaultWriteObject();
    }
    
    public static void clearBackup() {
        processorBackup.clear();
    }
    
    static void clearBackup(String value) {
        processorBackup.remove(value);
    }

    public void setProcessor(Processor processor) {
        this.processor = processor;
    }
    
    public void removeListener() {
        processor.removeListener(this);
    }
    
    public static mxProcessor findProcessor(Processor p, mxGraphModel model) {
        Map cells = model.getCells();
        for(Object cell : cells.values()) {
            if(cell instanceof mxProcessor) {
                if(((mxProcessor)cell).getProcessor() == p) {
                    return (mxProcessor)cell;
                }
            }
        }
        
        return null;
    }
} 
