package dans.algorithm.examples;

import dans.algorithm.Algorithm;
import dans.algorithm.Message;

/**
 * This algorithm elects a leader in a bidirectional ring network. It is assumed
 * that a bidirectional ring network of at least two processors exists and that
 * each processor has a link to the neighbour on its left and right in the ring.
 * It is further assumed that the processors do not know which neighbour is their
 * left neighbour and which is their right neighbour just that they know the IDs
 * of two neighbours. It is required that all processors in the network have
 * numerical IDs.
 * 
 * 
 * @author Daniel Servos
 * @version 0.1beta
 */
public class BidirectionalLeaderElection extends Algorithm {

    @Override
    public Object algorithm() {
        int id = Integer.parseInt(getID());
        String msgA = getID();
        String msgB = getID();
        String status = "UNKOWN";
        String neighbors[] = getNeighbors();
        int leader = -1;
        
        while(doMainLoop()) {
            if(msgA != null) {
                send(neighbors[0], msgA);
            }
            
            if(msgB != null) {
                send(neighbors[1], msgB);
            }
            
            if(leader >= 0) {
                return leader;
            }
            
            msgA = null;
            msgB = null;
            
            Message in;
            while((in = receive()) != null) {
                String m = in.message();
                String from = in.from();
                String splited[] = m.split(",");

                if(m.startsWith("END")) {
                    leader = Integer.parseInt(splited[1]);
                    
                    if(from.equals(neighbors[0])) {
                        if(leader == Integer.parseInt(neighbors[1])) return leader;
                        msgB = m;
                    } else {
                        if(leader == Integer.parseInt(neighbors[0])) return leader;
                        msgA = m;
                    }
                } else {
                    int i = in.toInt();
                    
                    if(i > id) {
                        status = "NOT LEADER";
                        
                        if(from.equals(neighbors[0])) {
                            msgB = m;
                        } else {
                            msgA = m;
                        }
                    } else if(i == id) {
                        status = "LEADER";
                        leader = id;
                        msgA = "END," + id;
                    }
                }
                
                display(status);
            }
        }
        
        return leader;
    }
    
}
