package dans.algorithm.examples;

import dans.algorithm.Algorithm;
import dans.algorithm.Message;
import java.util.ArrayList;

/**
 * This algorithm broadcasts the message "Hello World" from the processor with
 * the ID "root" to all other processors in the network. It is assumed that all
 * links are bidirectional and that the network graph is strongly connected. It
 * is required that one and only one processor be given the ID "root".
 * 
 * @author Daniel Servos
 * @version 0.1beta
 */
public class Broadcast extends Algorithm {
    @Override
    public Object algorithm() {
        String parent = null;
        String message = null;
        int numNeighbors = getNeighbors().length;
        ArrayList<String> acked = new ArrayList<>();
        
        if(getID().equalsIgnoreCase("root")) {
            message = "Hello World";
            for(String nid : getNeighbors()) {
                send(nid, "BROADCAST," + message);
            }
        }
        
        while(doMainLoop()) {            
            Message m;
            while((m = receive()) != null) {
                if(m.message().startsWith("BROADCAST,")) {
                    if(message == null) {
                        message = m.message().split(",")[1];
                        parent = m.from();
                        for(String nid : getNeighbors()) {
                            if(!nid.equals(parent)) {
                                send(nid,  m.message());
                            }
                        }
                    } else {
                        send(m.from(),"ACK");
                    }
                } else if(m.message().equals("ACK")) {
                    acked.add(m.from());
                }
            }
            
            if(getID().equalsIgnoreCase("root")) {
                if(acked.size() >= numNeighbors) {
                    return message;
                }
            } else {
                if(parent != null && acked.size() >= numNeighbors - 1) {
                    send(parent, "ACK");
                    return message;
                } 
            }
            
        }
        
        return message;
    }
    
}
