package dans.algorithm.examples;

import dans.algorithm.Algorithm;
import dans.algorithm.Message;

/**
 * Given a BFS tree of a network, this algorithm finds the sum of the processor's
 * IDs in the network. It is assumed that the BFS tree is represented by laying
 * the network graph out in a tree with only one directional link leaving each
 * processor (other than the root). Only the root processor will find the correct
 * sum, any other processor will simply have the sum of its self and its children.
 * It is required that all processor IDs be numerical.
 * 
 * @author Daniel Servos
 * @version 0.1beta
 */
public class TreeSumID extends Algorithm {

    @Override
    public Object algorithm() {
        String parents[] = getNeighbors();
        String childern[] = getSources();
        int id = Integer.parseInt(getID());
        String msg = null;
        int sum = id;
        int heardfrom = 0;
        
        
        if(childern.length == 0) {
            msg = getID();
        } else if(parents.length == 0 && childern.length == 0) {
            return sum;
        }
        
        while(doMainLoop()) {
            if(msg != null) {
                for(String pid : parents) {
                    send(new Message(pid, msg));
                }
                
                return sum;
            }
            
            msg = null;
            
            Message in;
            while((in = receive()) != null) {
                int i = in.toInt();
                sum += i;

                heardfrom++;
            }
            
            if(heardfrom  >= childern.length) {
                msg = sum + "";
            }
        }
        
        return sum;
    }
    
}
