package dans.network;

import dans.util.Log;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 *
 * @author Dan
 */
public class MessageInQueue {
    private NetworkState state;
    private final Object lock = new Object();
    private final ConcurrentLinkedQueue<BackEndMessage> in = new ConcurrentLinkedQueue<>();
    
    MessageInQueue(NetworkState ns) {
        state = ns;
    }

    void transfer(BackEndMessage bm) {
        synchronized(lock) {
            bm.receivedTick = state.getTick();
            in.add(bm);
            lock.notify();
        }
        
        state.incrementNetworkStat("Messages Sent", this);
        state.incrementNetworkStat("Bytes Sent", bm.size(), this);
        
        Log.debug("Receiving message \"" + bm + "\" from " + bm.fromID, "P:" + bm.toID + "#" + bm.port);
    }
        
    public BackEndMessage receiveWait() {
        synchronized(lock) {
            while(in.isEmpty()) {
                try {
                    lock.wait();
                } catch (InterruptedException ex) {}
            }
            
            return in.poll();
        }
    }
    
    public BackEndMessage receive() {
        synchronized(lock) {
            return in.poll();
        }
    }
}
