package dans.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.logging.Level;
import java.util.logging.Logger;
import dans.algorithm.Algorithm;
import java.io.File;

/**
 *
 * @author Dan
 */
public class AlgorithmLoader extends ClassLoader{
    public AlgorithmLoader(ClassLoader parent) {
        super(parent);
    }
    
    //Based on code from http://tutorials.jenkov.com/java-reflection/dynamic-class-loading-reloading.html
    public Class loadClass(File file) {
        URL classFile;      
        try {
            classFile = file.toURI().toURL();
        } catch (MalformedURLException ex) {
            Log.error("Bad class filename/path.", ex);
            return null;
        }

        try {
            URLConnection connection = classFile.openConnection();
            InputStream input = connection.getInputStream();
            ByteArrayOutputStream buffer = new ByteArrayOutputStream();
            int data = input.read();

            while(data != -1) {
                buffer.write(data);
                data = input.read();
            }

            input.close();

            byte[] classData = buffer.toByteArray();

            return defineClass(null, classData, 0, classData.length);

        } catch (IOException e) {
            Log.error("Failed to load class.", e);
        }

        return null;
    }
    
    public static Class loadAlgorithm(File file) {
        Log.debug("Attempting to load Algorithm class from " + file.toString());
        AlgorithmLoader al = new AlgorithmLoader(AlgorithmLoader.class.getClassLoader());
        Class c = al.loadClass(file);
        if(c != null) {
             if(Algorithm.class.isAssignableFrom(c)) {
                return c;
             } else {
                 Log.error("Class " + c.getName() + " is not a child of the Algorithm abstract class.");
             }
        }
        
        return null;
    }
}
